import { faker } from '@faker-js/faker';
import { E2EBenutzerHelper } from 'apps/admin-e2e/src/helper/benutzer/benutzer.helper';
import { E2EBenutzerVerifier } from 'apps/admin-e2e/src/helper/benutzer/benutzer.verifier';
import { getCypressEnv, interceptWithResponse, waitOfInterceptor } from 'apps/admin-e2e/src/support/cypress-helper';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { AdminUserE2E, HttpMethodE2E, SnackbarMessagesE2E } from '../../../model/util';
import { contains, notExist } from '../../../support/cypress.util';
import { loginAsAriane, loginByUsernameAndPassword } from '../../../support/user-util';

describe('Benutzer anlegen', () => {
  const benutzerHelper: E2EBenutzerHelper = new E2EBenutzerHelper();
  const benutzerVerifier: E2EBenutzerVerifier = new E2EBenutzerVerifier();

  const snackBar: SnackBarE2EComponent = new SnackBarE2EComponent();

  const newUserPassword: string = 'TestTestTest';
  const newAdminUser: AdminUserE2E = {
    vorname: 'Theo',
    nachname: 'Testuser',
    username: 'testtheo',
    email: 'theo.test@ozg-sh.de',
    isAdmin: true,
    isUser: true,
    organisationseinheiten: [],
  };

  const newRegularUser: AdminUserE2E = {
    vorname: 'Max',
    nachname: 'Mustermann',
    username: 'maxmust',
    email: 'max.mustermann@ozg-sh.de',
    isUser: true,
    organisationseinheiten: [],
  };

  before(() => {
    loginAsAriane();
  });

  it('should show error snackbar after save and receiving error response', () => {
    const interceptor: string = 'postUser';
    const url: string = getCypressEnv('keycloakUrl') + '/admin/realms/' + getCypressEnv('keycloakRealm') + '/users';
    const errorCode: number = 500;
    const errorBody: string = 'Internal Server Error';
    interceptWithResponse(HttpMethodE2E.POST, url, { errorCode, errorBody }).as(interceptor);
    benutzerHelper.openNewBenutzerPage();

    benutzerHelper.addBenutzer(newAdminUser);
    benutzerHelper.saveBenutzer();

    waitOfInterceptor(interceptor).then(() => {
      contains(snackBar.getMessage(), SnackbarMessagesE2E.NUTZER_FEHLGESCHLAGEN);
      snackBar.getCloseButton().click();
      notExist(snackBar.getMessage());
    });
  });

  it('should show snackbar after save', () => {
    benutzerHelper.openNewBenutzerPage();

    benutzerHelper.addBenutzer(newAdminUser);
    benutzerHelper.saveBenutzer();

    contains(snackBar.getMessage(), SnackbarMessagesE2E.NUTZER_ANGELEGT);
    snackBar.getCloseButton().click();
  });

  it('should show created user in list', () => {
    benutzerVerifier.verifyUserInList(newAdminUser);
  });

  it('should remove benutzer', () => {
    benutzerHelper.deleteBenutzer(newAdminUser.username);
    benutzerVerifier.verifyUserNotInList(newAdminUser.username);
  });

  describe('newly created admin user', () => {
    it('should create new admin user', () => {
      benutzerHelper.openNewBenutzerPage();

      benutzerHelper.addBenutzer(newAdminUser);
      benutzerHelper.saveBenutzer();

      benutzerVerifier.verifyUserInList(newAdminUser);
    });

    it('should create new regular user', () => {
      benutzerHelper.openNewBenutzerPage();

      benutzerHelper.addBenutzer(newRegularUser);
      benutzerHelper.saveBenutzer();

      benutzerVerifier.verifyUserInList(newRegularUser);
    });

    it('should modify user', () => {
      const userWithChangedFirstName = { ...newRegularUser, vorname: faker.person.firstName() };

      benutzerHelper.activateUser(newAdminUser.username, newUserPassword).then(() => {
        loginByUsernameAndPassword(newAdminUser.username, newUserPassword);
        benutzerHelper.openBenutzerPage(newRegularUser.username);

        benutzerHelper.editBenutzer(userWithChangedFirstName);
        benutzerHelper.saveBenutzer();

        benutzerVerifier.verifyUserInList(userWithChangedFirstName);
      });
    });

    it('should have rights to delete user', () => {
      loginByUsernameAndPassword(newAdminUser.username, newUserPassword);
      benutzerHelper.openBenutzerListPage();

      benutzerHelper.deleteBenutzer(newRegularUser.username);

      benutzerVerifier.verifyUserNotInList(newRegularUser.username);
    });

    it('should delete user', () => {
      loginAsAriane();

      benutzerHelper.deleteBenutzer(newAdminUser.username);
    });
  });

  describe('when filling formular', () => {
    it('should show validation errors on empty formular', () => {
      benutzerHelper.openNewBenutzerPage();
      benutzerHelper.saveBenutzer();

      benutzerVerifier.verifyEmptyVornameError();
      benutzerVerifier.verifyEmptyNameError();
      benutzerVerifier.verifyUsernameLengthError();
      benutzerVerifier.verifyInvalidEmailError();
      benutzerVerifier.verifyEmptyRollenError();
    });

    it('should hide Vorname validation error on filled', () => {
      benutzerHelper.editVorname('Max');
      benutzerVerifier.verifyVornameHasNoError();
      benutzerVerifier.verifyEmptyNameError();
      benutzerVerifier.verifyUsernameLengthError();
      benutzerVerifier.verifyInvalidEmailError();
      benutzerVerifier.verifyEmptyRollenError();
    });

    it('should hide Nachname validation error on filled', () => {
      benutzerHelper.editNachname('Mustermann');
      benutzerVerifier.verifyVornameHasNoError();
      benutzerVerifier.verifyLastNameHasNoError();
      benutzerVerifier.verifyUsernameLengthError();
      benutzerVerifier.verifyInvalidEmailError();
      benutzerVerifier.verifyEmptyRollenError();
    });

    it('should hide Benutzername validation error on filled', () => {
      benutzerHelper.editBenutzername('Max');
      benutzerVerifier.verifyVornameHasNoError();
      benutzerVerifier.verifyLastNameHasNoError();
      benutzerVerifier.verifyUsernameHasNoError();
      benutzerVerifier.verifyInvalidEmailError();
      benutzerVerifier.verifyEmptyRollenError();
    });

    it('should hide Email validation error on filled', () => {
      benutzerHelper.editEmail('max@max.local');
      benutzerVerifier.verifyVornameHasNoError();
      benutzerVerifier.verifyLastNameHasNoError();
      benutzerVerifier.verifyUsernameHasNoError();
      benutzerVerifier.verifyEmailHasNoError();
      benutzerVerifier.verifyEmptyRollenError();
    });

    it('should hide Rollen validation error on filled', () => {
      benutzerHelper.addUserRole();
      benutzerVerifier.verifyVornameHasNoError();
      benutzerVerifier.verifyLastNameHasNoError();
      benutzerVerifier.verifyUsernameHasNoError();
      benutzerVerifier.verifyEmailHasNoError();
      benutzerVerifier.verifyRollenHasNoError();
    });
  });

  describe('after formular submission', () => {
    it('should have user created', () => {
      benutzerHelper.openNewBenutzerPage();

      benutzerHelper.addBenutzer(newAdminUser);
      benutzerHelper.saveBenutzer();

      benutzerVerifier.verifyUserInList(newAdminUser);
    });

    it('should show validation error on existing email', () => {
      benutzerHelper.openNewBenutzerPage();
      benutzerHelper.addBenutzer(newAdminUser);
      benutzerHelper.saveBenutzer();

      benutzerVerifier.verifyEmailExistsError();
    });

    it('should show validation error on existing username', () => {
      benutzerHelper.openNewBenutzerPage();
      benutzerHelper.addBenutzer({ ...newAdminUser, email: faker.internet.email() + '.local' });
      benutzerHelper.saveBenutzer();

      benutzerVerifier.verifyUsernameExistsError();
    });

    it('should remove benutzer', () => {
      benutzerHelper.openBenutzerPage(newAdminUser.username);
      benutzerHelper.deleteBenutzer(newAdminUser.username);

      benutzerVerifier.verifyUserNotInList(newAdminUser.username);
    });
  });
});
