/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ConfigurationService } from '@admin-client/configuration-shared';
import { ROUTES } from '@admin-client/shared';
import { ApiRootService } from '@alfa-client/api-root-shared';
import { LinkRelationName, mapToResource, StateResource } from '@alfa-client/tech-shared';
import { inject } from '@angular/core';
import { ActivatedRouteSnapshot, CanActivateFn, Router, UrlTree } from '@angular/router';
import { AuthenticationService } from '@authentication';
import { hasLink, Resource } from '@ngxp/rest';
import { filter, map, Observable, of } from 'rxjs';
import { GuardData } from './app.routes';

import * as Guard from './app.guard';

export const apiRootGuard: CanActivateFn = (route: ActivatedRouteSnapshot): Observable<true | UrlTree> => {
  const apiRootService: ApiRootService = inject(ApiRootService);
  return Guard.evaluate(apiRootService.getApiRoot(), getLinkRelationName(route));
};

export const configurationGuard: CanActivateFn = (route: ActivatedRouteSnapshot): Observable<true | UrlTree> => {
  const configurationService: ConfigurationService = inject(ConfigurationService);
  return Guard.evaluate(configurationService.get(), getLinkRelationName(route));
};

export function evaluate(
  stateResource$: Observable<StateResource<Resource>>,
  linkRelationName: LinkRelationName,
): Observable<true | UrlTree> {
  const authService = inject(AuthenticationService);
  const router = inject(Router);

  if (!authService.isLoggedIn()) return of(true);

  return stateResource$.pipe(
    filter((stateResource: StateResource<Resource>): boolean => stateResource.loaded),
    mapToResource<Resource>(),
    map((resource: Resource): true | UrlTree => Guard.evaluateResource(resource, linkRelationName, router)),
  );
}

function getLinkRelationName(route: ActivatedRouteSnapshot): string {
  return (<GuardData>route.data).linkRelName;
}

export function evaluateResource(resource: Resource, linkRelationName: LinkRelationName, router: Router): true | UrlTree {
  return hasLink(resource, linkRelationName) ? true : redirectToUnavailable(router);
}

function redirectToUnavailable(router: Router): UrlTree {
  return router.createUrlTree(['/' + ROUTES.UNAVAILABLE]);
}
