/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { contains, exist, notContains } from 'apps/alfa-e2e/src/support/cypress.util';
import { VorgangDetailHeaderE2EComponent } from '../../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangFormularDatenE2EComponent } from '../../../components/vorgang/vorgang-formular.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { loginAsSabine } from '../../../support/user-util';
import {
  buildVorgang,
  createVorgang,
  initVorgaenge,
  objectIds,
} from '../../../support/vorgang-util';
import { VorgangSubnavigationE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-subnavigation';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Detailansicht with formData', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangDatenFormular: VorgangFormularDatenE2EComponent =
    vorgangPage.getFormularDatenContainer();

  const vorgangHeader: VorgangDetailHeaderE2EComponent = vorgangPage.getVorgangDetailHeader();
  const vorgangSubnavigation: VorgangSubnavigationE2EComponent = new VorgangSubnavigationE2EComponent();

  const ort: string = 'Ort';
  const sh_strasse: string = 'Strasse';
  const sh_hausnummer: string = 'Hausnummer';
  const emailadresse: string = 'E-Mail Adresse';
  const postleitzahl: string = 'PLZ';

  const vorgang: VorgangE2E = { ...createVorgang(), name: 'TestVorgang' };
  const vorgangWithLabels: VorgangE2E = createVorgangWithLabels();

  before(() => {
    initVorgaenge([vorgang, vorgangWithLabels]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  function createVorgangWithLabels(): VorgangE2E {
    const vorgang = buildVorgang(objectIds[0], 'TestVorgangWithLabels');
    return {
      ...vorgang,
      eingangs: [
        {
          ...vorgang.eingangs[0],
          formData: createFormDataWithLabels(),
        },
      ],
    };
  }

  function createFormDataWithLabels(): FormData {
    return {
      ...vorgang.eingangs[0].formData,
      _kopControlData: {
        metaData: true,
        labels: { ort, sh_strasse, sh_hausnummer },
      },
      empfangendestelle: createEmpfangendestelleWithLabels(),
    };
  }

  function createEmpfangendestelleWithLabels(): any {
    return {
      ...vorgang.eingangs[0].formData.empfangendestelle,
      _kopControlData: {
        labels: {
          emailadresse,
          postleitzahl,
        },
      },
    };
  }

  after(() => {
    dropCollections();
  });

  describe('navigate to vorgang detail (without labels)', () => {
    describe('click on vorgang in list', () => {
      it('should show header', () => {
        vorgangList.getListItem(vorgang.name).getRoot().click();
        waitForSpinnerToDisappear();

        exist(vorgangHeader.getRoot());
      });
    });

    describe('click on "Antragsdaten" tab', () => {
      it('should show on click on tab', () => {
        vorgangDatenFormular.getAntragdatenTab().click();

        exist(vorgangDatenFormular.getAntragdaten());
      });

      it('should show "sh_hausnummer"', () => {
        contains(vorgangDatenFormular.getRoot(), 'sh_hausnummer');
      });

      it('should show "sh_strasse"', () => {
        contains(vorgangDatenFormular.getRoot(), 'sh_strasse');
      });
    });
  });

  describe('navigate to vorgang detail (with labels)', () => {
    describe('click on vorgang in list', () => {
      it('should show header', () => {
        vorgangSubnavigation.back();
        vorgangList.getListItem(vorgangWithLabels.name).getRoot().click();
        waitForSpinnerToDisappear();

        exist(vorgangHeader.getRoot());
      });
    });

    describe('click on "Antragsdaten" tab', () => {
      it('should show on click on tab', () => {
        vorgangDatenFormular.getAntragdatenTab().click();

        exist(vorgangDatenFormular.getAntragdaten());
      });

      it('should not show "_kopControlData', () => {
        notContains(vorgangDatenFormular.getRoot(), '_kopControlData');
      });

      it('should show "Hausnummer"', () => {
        contains(vorgangDatenFormular.getRoot(), sh_hausnummer);
      });

      it('should show "Strasse"', () => {
        contains(vorgangDatenFormular.getRoot(), sh_strasse);
      });

      it('should show "Ort"', () => {
        contains(vorgangDatenFormular.getRoot(), ort);
      });
    });

  });
});
