/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { NavigationE2EComponent } from 'apps/alfa-e2e/src/components/navigation/navigation.e2e.component';
import { VorgangListItemE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-item.e2e.component';
import { VorgangListE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-list.e2e.component';
import { VorgangSearchE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-search.e2e.component';
import {
  VorgangViewE2EComponent,
  VorgangViewsE2EComponent,
} from 'apps/alfa-e2e/src/components/vorgang/vorgang-views.e2e.component';
import { ClientAttributeNameE2E, VorgangE2E, VorgangStatusE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { isOdsButtonToggleChecked, isOdsButtonToggleNotChecked } from 'apps/alfa-e2e/src/support/angular.util';
import { dropCollections, dropSearchIndex } from 'apps/alfa-e2e/src/support/cypress-helper';
import { containClass, contains, exist, haveLength, haveText, notContainClass } from 'apps/alfa-e2e/src/support/cypress.util';
import { createDateToday } from 'apps/alfa-e2e/src/support/tech.util';
import { getUserSabineId, initUsermanagerUsers, loginAsSabine } from 'apps/alfa-e2e/src/support/user-util';
import {
  buildVorgang,
  createAlfaClientAttributes,
  createNextWiedervorlageFristClientAttribute,
  createVorgang,
  initSearchIndex,
  initVorgaenge,
  objectIds,
} from 'apps/alfa-e2e/src/support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang views and filter', () => {
  const mainPage: MainPage = new MainPage();

  const header: HeaderE2EComponent = mainPage.getHeader();
  const vorgangSearch: VorgangSearchE2EComponent = header.getVorgangSearch();

  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const views: VorgangViewsE2EComponent = mainPage.getViews();
  const neuView: VorgangViewE2EComponent = views.getNeu();
  const angenommenView: VorgangViewE2EComponent = views.getAngenommen();
  const inBearbeitungView: VorgangViewE2EComponent = views.getInBearbeitung();
  const beschiedenView: VorgangViewE2EComponent = views.getBeschieden();
  const abgeschlossenView: VorgangViewE2EComponent = views.getAbgeschlossen();
  const weitergeleitetView: VorgangViewE2EComponent = views.getWeitergeleitet();
  const verworfenView: VorgangViewE2EComponent = views.getVerworfen();
  const zuLoeschenView: VorgangViewE2EComponent = views.getZuLoeschen();
  const ungelesenView: VorgangViewE2EComponent = views.getUngelesen();
  const wiedervorlagenView: VorgangViewE2EComponent = views.getWiedervorlagen();
  const vorgangListView: VorgangViewE2EComponent = views.getVorgangList();
  const sucheView: VorgangViewE2EComponent = views.getSuche();

  const ALL_VIEW_COMPONENTS: VorgangViewE2EComponent[] = [
    neuView,
    angenommenView,
    inBearbeitungView,
    beschiedenView,
    abgeschlossenView,
    weitergeleitetView,
    verworfenView,
    zuLoeschenView,
    ungelesenView,
    wiedervorlagenView,
    vorgangListView,
    sucheView,
  ];

  const navigation: NavigationE2EComponent = mainPage.getNavigation();

  const vorgangNeuAssigned: VorgangE2E = {
    ...createVorgang(),
    name: 'VorgangNeuAssigned',
    status: VorgangStatusE2E.NEU,
    assignedTo: getUserSabineId(),
  };
  const vorgangNeuUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'VorgangNeuUnassigned'),
    status: VorgangStatusE2E.NEU,
  };

  const vorgangAngenommenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[1], 'VorgangAngenommenAssigned'),
    status: VorgangStatusE2E.ANGENOMMEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangAngenommenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[2], 'VorgangAngenommenUnassigned'),
    status: VorgangStatusE2E.ANGENOMMEN,
    clientAttributes: createAlfaClientAttributes(
      ClientAttributeNameE2E.NEXT_WIEDERVORLAGE_FRIST,
      createNextWiedervorlageFristClientAttribute(createDateToday()),
    ),
  };

  const vorgangInBearbeitungAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[3], 'VorgangInBearbeitungAssigned'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
    assignedTo: getUserSabineId(),
  };
  const vorgangInBearbeitungUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[4], 'VorgangInBearbeitungUnassigned'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const vorgangBeschiedenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[5], 'VorgangBeschiedenAssigned'),
    status: VorgangStatusE2E.BESCHIEDEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangBeschiedenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[6], 'VorgangBeschiedenUnassigned'),
    status: VorgangStatusE2E.BESCHIEDEN,
  };

  const vorgangAbgeschlossenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[7], 'VorgangAbgeschlossenAssigned'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
    assignedTo: getUserSabineId(),
    clientAttributes: createAlfaClientAttributes(
      ClientAttributeNameE2E.NEXT_WIEDERVORLAGE_FRIST,
      createNextWiedervorlageFristClientAttribute(createDateToday()),
    ),
  };

  const vorgangAbgeschlossenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[8], 'VorgangAbgeschlossenUnassigned'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
  };

  const vorgangWeitergeleitetAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[13], 'VorgangWeitergeleitetAssigned'),
    status: VorgangStatusE2E.WEITERGELEITET,
    assignedTo: getUserSabineId(),
  };
  const vorgangWeitergeleitetUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[14], 'VorgangWeitergeleitetUnassigned'),
    status: VorgangStatusE2E.WEITERGELEITET,
  };

  const vorgangVerworfenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[9], 'VorgangVerworfenAssigned'),
    status: VorgangStatusE2E.VERWORFEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangVerworfenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[10], 'VorgangVerworfenUnassigned'),
    status: VorgangStatusE2E.VERWORFEN,
  };

  const vorgangZuLoeschenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[11], 'VorgangZuLoeschenAssigned'),
    status: VorgangStatusE2E.ZU_LOESCHEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangZuLoeschenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[12], 'VorgangZuLoeschenUnassigned'),
    status: VorgangStatusE2E.ZU_LOESCHEN,
  };

  const vorgangNeuAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangNeuAssigned.name);
  const vorgangNeuUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangNeuUnassigned.name);

  const vorgangAngenommenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangAngenommenAssigned.name);
  const vorgangAngenommenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangAngenommenUnassigned.name,
  );

  const vorgangInBearbeitungAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangInBearbeitungAssigned.name,
  );
  const vorgangInBearbeitungUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangInBearbeitungUnassigned.name,
  );

  const vorgangBeschiedenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangBeschiedenAssigned.name);
  const vorgangBeschiedenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangBeschiedenUnassigned.name,
  );

  const vorgangAbgeschlossenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangAbgeschlossenAssigned.name,
  );
  const vorgangAbgeschlossenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangAbgeschlossenUnassigned.name,
  );

  const vorgangWeitergeleitetAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangWeitergeleitetAssigned.name,
  );
  const vorgangWeitergeleitetUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangWeitergeleitetUnassigned.name,
  );

  const vorgangVerworfenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangVerworfenAssigned.name);
  const vorgangVerworfenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangVerworfenUnassigned.name);

  const vorgangZuLoeschenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangZuLoeschenAssigned.name);
  const vorgangZuLoeschenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangZuLoeschenUnassigned.name,
  );

  const LIST_SIZE: number = 16;

  before(() => {
    const vorgaenge: VorgangE2E[] = [
      vorgangNeuAssigned,
      vorgangNeuUnassigned,
      vorgangAngenommenAssigned,
      vorgangAngenommenUnassigned,
      vorgangInBearbeitungAssigned,
      vorgangInBearbeitungUnassigned,
      vorgangBeschiedenAssigned,
      vorgangBeschiedenUnassigned,
      vorgangAbgeschlossenAssigned,
      vorgangAbgeschlossenUnassigned,
      vorgangWeitergeleitetAssigned,
      vorgangWeitergeleitetUnassigned,
      vorgangVerworfenAssigned,
      vorgangVerworfenUnassigned,
      vorgangZuLoeschenAssigned,
      vorgangZuLoeschenUnassigned,
    ];
    initVorgaenge(vorgaenge);

    initSearchIndex(vorgaenge);
    initUsermanagerUsers();

    loginAsSabine();
  });

  after(() => {
    dropCollections();
    dropSearchIndex();
  });

  describe('Initial', () => {
    describe('on load', () => {
      it('should show vorgaenge in vorgangList', () => {
        exist(vorgangList.getRoot());

        haveLength(vorgangList.getItems(), LIST_SIZE);
      });
    });

    describe('filter', () => {
      it('Alle should be selected', () => {
        isOdsButtonToggleChecked(navigation.getAlleFilter());
      });

      it('Meine Vorgänge should not be selected', () => {
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('Nicht zugewiesen should not be selected', () => {
        isOdsButtonToggleNotChecked(navigation.getUnassignedFilter());
      });
    });

    describe('view', () => {
      it('vorgangList should be selected', () => {
        isSelected(vorgangListView.getRoot());
      });

      it('other should be unselected', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(weitergeleitetView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      describe('count for', () => {
        it('neu', () => {
          contains(neuView.getCount(), '2');
        });

        it('angenommen', () => {
          contains(angenommenView.getCount(), '2');
        });

        it('in_bearbeitung', () => {
          contains(inBearbeitungView.getCount(), '2');
        });

        it('beschieden', () => {
          contains(beschiedenView.getCount(), '2');
        });

        it('abgeschlossen', () => {
          contains(abgeschlossenView.getCount(), '2');
        });

        it('weitergeleitet', () => {
          contains(abgeschlossenView.getCount(), '2');
        });

        it('verworfen', () => {
          contains(verworfenView.getCount(), '2');
        });

        it('zu löschen', () => {
          contains(zuLoeschenView.getCount(), '2');
        });

        it('wiedervorlagen', () => {
          contains(wiedervorlagenView.getCount(), '2');
        });
      });
    });
  });

  describe('Select view', () => {
    describe('Neu', () => {
      testAlleFilterViewSelection(neuView, [vorgangNeuAssignedInList, vorgangNeuUnassignedInList]);
    });

    describe('Angenommen', () => {
      testAlleFilterViewSelection(angenommenView, [vorgangAngenommenAssignedInList, vorgangAngenommenUnassignedInList]);
    });

    describe('In Bearbeitung', () => {
      testAlleFilterViewSelection(inBearbeitungView, [vorgangInBearbeitungAssignedInList, vorgangInBearbeitungUnassignedInList]);
    });

    describe('Beschieden', () => {
      testAlleFilterViewSelection(beschiedenView, [vorgangBeschiedenAssignedInList, vorgangBeschiedenUnassignedInList]);
    });

    describe('Abgeschlossen', () => {
      testAlleFilterViewSelection(abgeschlossenView, [vorgangAbgeschlossenAssignedInList, vorgangAbgeschlossenUnassignedInList]);
    });

    describe('Weitergeleitet', () => {
      testAlleFilterViewSelection(weitergeleitetView, [
        vorgangWeitergeleitetAssignedInList,
        vorgangWeitergeleitetUnassignedInList,
      ]);
    });

    describe('Verworfen', () => {
      testAlleFilterViewSelection(verworfenView, [vorgangVerworfenAssignedInList, vorgangVerworfenUnassignedInList]);
    });

    describe('Zu Löschen', () => {
      testAlleFilterViewSelection(zuLoeschenView, [vorgangZuLoeschenAssignedInList, vorgangZuLoeschenUnassignedInList]);
    });

    describe('Wiedervorlagen', () => {
      testAlleFilterViewSelection(wiedervorlagenView, [vorgangAngenommenUnassignedInList, vorgangAbgeschlossenAssignedInList]);
    });

    function testAlleFilterViewSelection(view: VorgangViewE2EComponent, listItems: VorgangListItemE2EComponent[]): void {
      testViewSelection(view);
      testOtherViewsAreUnselected(view);
      testAlleFilterIsSelected();
      testVorgangListItem(listItems);
    }

    function testViewSelection(view: VorgangViewE2EComponent): void {
      it('should set selected after click', () => {
        view.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(view.getRoot());
      });
    }

    function testOtherViewsAreUnselected(selectedView: VorgangViewE2EComponent): void {
      it('should unselect other views', () => {
        ALL_VIEW_COMPONENTS.filter((view: VorgangViewE2EComponent) => view != selectedView).forEach(
          (view: VorgangViewE2EComponent) => isNotSelected(view.getRoot()),
        );
      });
    }

    function testAlleFilterIsSelected(): void {
      it('should keep filter selection', () => {
        isOdsButtonToggleChecked(navigation.getAlleFilter());
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isOdsButtonToggleNotChecked(navigation.getUnassignedFilter());
      });
    }

    function testVorgangListItem(listItems: VorgangListItemE2EComponent[]): void {
      it('should filter List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE - 14);
        listItems.forEach((item: VorgangListItemE2EComponent) => exist(item.getRoot()));
      });
    }

    describe('VorgangList', () => {
      it('should set selected after click', () => {
        vorgangListView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(vorgangListView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(weitergeleitetView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isOdsButtonToggleChecked(navigation.getAlleFilter());
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isOdsButtonToggleNotChecked(navigation.getUnassignedFilter());
      });

      it('should show full List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE);
      });
    });

    describe('Suche', () => {
      it('should set selected after click', () => {
        sucheView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(sucheView.getRoot());
      });

      testOtherViewsAreUnselected(sucheView);
      testAlleFilterIsSelected();

      it('should show message', () => {
        haveText(vorgangList.getEmptySucheMessage(), 'Bitte geben Sie einen Suchbegriff ein.');
      });
    });
  });

  describe('Select Filter', () => {
    describe('Nicht zugewiesen', () => {
      it('should set selected after click', () => {
        navigation.getUnassignedFilter().click();
        waitForSpinnerToDisappear();

        isOdsButtonToggleChecked(navigation.getUnassignedFilter());
      });

      it('should unselect Meine Vorgänge', () => {
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep Alle unselected', () => {
        isOdsButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(weitergeleitetView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });

      it('should show message', () => {
        haveText(vorgangList.getEmptySucheMessage(), 'Bitte geben Sie einen Suchbegriff ein.');
      });
    });

    describe('Meine Vorgänge', () => {
      it('should set selected after click', () => {
        navigation.getMeineVorgaengeFilter().click();
        waitForSpinnerToDisappear();

        isOdsButtonToggleChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should unselect Alle', () => {
        isOdsButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });

      it('should show message', () => {
        haveText(vorgangList.getEmptySucheMessage(), 'Bitte geben Sie einen Suchbegriff ein.');
      });
    });
  });

  describe('Suche', () => {
    describe('on execute/enter with Meine Vorgänge filter selected', () => {
      it('should show filtered list', () => {
        doSearch('Angenommen');
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 1);
        exist(vorgangAngenommenAssignedInList.getRoot());
      });

      it('should keep filter selection', () => {
        isOdsButtonToggleChecked(navigation.getMeineVorgaengeFilter());
        isOdsButtonToggleNotChecked(navigation.getAlleFilter());
        isOdsButtonToggleNotChecked(navigation.getUnassignedFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Alle filter selection', () => {
      it('should show filtered list', () => {
        navigation.getAlleFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 2);
        exist(vorgangAngenommenAssignedInList.getRoot());
        exist(vorgangAngenommenUnassignedInList.getRoot());
      });

      it('should select Alle filter', () => {
        isOdsButtonToggleChecked(navigation.getAlleFilter());
      });

      it('should unselect Meine Vorgänge filter', () => {
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Nicht zugewiesen filter selection', () => {
      it('should show filtered list', () => {
        navigation.getUnassignedFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 1);
        exist(vorgangAngenommenUnassignedInList.getRoot());
      });

      it('should show Nicht zugewiesen as selected', () => {
        isOdsButtonToggleChecked(navigation.getUnassignedFilter());
      });

      it('should unselect Alle filter', () => {
        isOdsButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep Meine Vorgänge unselected', () => {
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on other search execution', () => {
      it('should show filtered list', () => {
        navigation.getAlleFilter().click();
        waitForSpinnerToDisappear();

        doSearch('VorgangA');
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 4);
        exist(vorgangAngenommenAssignedInList.getRoot());
        exist(vorgangAngenommenUnassignedInList.getRoot());
        exist(vorgangAbgeschlossenAssignedInList.getRoot());
        exist(vorgangAbgeschlossenUnassignedInList.getRoot());
      });

      it('should keep filter selection', () => {
        isOdsButtonToggleChecked(navigation.getAlleFilter());
        isOdsButtonToggleNotChecked(navigation.getUnassignedFilter());
      });

      it('should unselect Meine Vorgänge filter', () => {
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Meine Vorgänge filter selection', () => {
      it('should show filtered list', () => {
        navigation.getMeineVorgaengeFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 2);
        exist(vorgangAngenommenAssignedInList.getRoot());
        exist(vorgangAbgeschlossenAssignedInList.getRoot());
      });

      it('should select', () => {
        isOdsButtonToggleChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should unselect Alle filter', () => {
        isOdsButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Nicht zugewiesen filter selection', () => {
      it('should show filtered list', () => {
        navigation.getUnassignedFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 2);
        exist(vorgangAngenommenUnassignedInList.getRoot());
        exist(vorgangAbgeschlossenUnassignedInList.getRoot());
      });

      it('should select', () => {
        isOdsButtonToggleChecked(navigation.getUnassignedFilter());
      });

      it('should unselect Meine Vorgänge filter', () => {
        isOdsButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep filter unselected', () => {
        isOdsButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });
  });

  function doSearch(searchString: string): void {
    vorgangSearch.doSearch(searchString);
  }

  function isNotSelected(element: Cypress.Chainable<Element>): void {
    notContainClass(element, 'selected');
  }

  function isSelected(element: Cypress.Chainable<Element>): void {
    containClass(element, 'selected');
  }
});
