/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  BinaryFileListLinkRel,
  BinaryFileListResource,
  BinaryFileResource,
  BinaryFileService,
} from '@alfa-client/binary-file-shared';
import {
  CommandOrder,
  CommandResource,
  CommandResourceService,
  CommandService,
  CreateCommandProps,
  getEffectedResourceUrl,
} from '@alfa-client/command-shared';
import { PostfachService } from '@alfa-client/postfach-shared';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
  EMPTY_STRING,
  getEmbeddedResources,
  ResourceListService,
  ResourceRepository,
  StateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { VorgangCommandService, VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { getUrl, LinkRel, ResourceUri } from '@ngxp/rest';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createProblemDetail } from 'libs/tech-shared/test/error';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { EMPTY, Observable, of } from 'rxjs';
import { createBinaryFileListResource, createBinaryFileResource } from '../../../binary-file-shared/test/binary-file';
import {
  createCommandErrorStateResource,
  createCommandResource,
  createCommandStateResource,
  createCreateCommandProps,
  createSuccessfullyDoneCommandStateResource,
} from '../../../command-shared/test/command';
import { createFile } from '../../../tech-shared/test/file';
import { singleCold, singleColdCompleted } from '../../../tech-shared/test/marbles';
import { createBescheid, createBescheidDocument, createBescheidListResource, createBescheidResource } from '../test/bescheid';
import { createDocumentResource } from '../test/document';
import { BescheidFacade } from './+state/bescheid.facade';
import { BescheidLinkRel } from './bescheid.linkrel';
import {
  Bescheid,
  BESCHEID_UPLOADED_ATTACHMENTS,
  BescheidDocument,
  BescheidListResource,
  BescheidResource,
  BescheidStatus,
  BescheidWizardStep,
  createEmptyBescheidDocument,
  createEmptyUploadInProgress,
  createInitialWizard,
} from './bescheid.model';
import { BescheidService } from './bescheid.service';
import { DocumentLinkRel } from './document.linkrel';
import { DocumentResource } from './document.model';

import {
  BescheidListResourceService,
  BescheidResourceService,
  buildCreateBescheidDocumentFromFileProps,
  buildUpdateBescheidCommandProps,
  createBescheidListResourceService,
  createBescheidResourceService,
} from '@alfa-client/bescheid-shared';
import { expect } from '@jest/globals';
import { cold } from 'jest-marbles';
import * as DateUtil from '../../../tech-shared/src/lib/date.util';

jest.mock('@alfa-client/bescheid-shared', () => {
  const originalModule = jest.requireActual('@alfa-client/bescheid-shared');

  return {
    __esModule: true,
    ...originalModule,
    createBescheidResourceService: jest.fn(),
    createBescheidListResourceService: jest.fn(),
    buildUpdateBescheidCommandProps: jest.fn(),
    buildCreateBescheidDocumentFromFileProps: jest.fn(),
  };
});

describe('BescheidService', () => {
  let service: BescheidService;
  let facade: Mock<BescheidFacade>;
  let vorgangService: Mock<VorgangService>;
  let resourceRepository: Mock<ResourceRepository>;
  let commandService: Mock<CommandService>;
  let vorgangCommandService: Mock<VorgangCommandService>;
  let binaryFileService: Mock<BinaryFileService>;
  let postfachService: Mock<PostfachService>;
  let bescheidResourceService: Mock<BescheidResourceService>;
  let bescheidListResourceService: Mock<BescheidListResourceService>;

  const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();
  const vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource> =
    createStateResource(vorgangWithEingangResource);
  const bescheidResource: BescheidResource = createBescheidResource();
  const commandStateResource: StateResource<CommandResource> = createCommandStateResource();

  beforeEach(() => {
    facade = mock(BescheidFacade);
    resourceRepository = mock(ResourceRepository);
    commandService = mock(CommandService);
    vorgangCommandService = mock(VorgangCommandService);
    vorgangService = mock(VorgangService);
    vorgangService.getVorgangWithEingang.mockReturnValue(of(vorgangWithEingangStateResource));
    binaryFileService = mock(BinaryFileService);
    postfachService = mock(PostfachService);
    bescheidResourceService = mock(CommandResourceService);
    bescheidListResourceService = mock(ResourceListService);
    (createBescheidResourceService as jest.Mock).mockReturnValue(bescheidResourceService);
    (createBescheidListResourceService as jest.Mock).mockReturnValue(bescheidListResourceService);

    TestBed.configureTestingModule({
      providers: [
        { provide: BescheidFacade, useValue: facade },
        { provide: VorgangService, useValue: vorgangService },
        { provide: ResourceRepository, useValue: resourceRepository },
        { provide: CommandService, useValue: commandService },
        { provide: VorgangCommandService, useValue: vorgangCommandService },
        { provide: BinaryFileService, useValue: binaryFileService },
        {
          provide: PostfachService,
          useValue: postfachService,
        },
        BescheidService,
      ],
    });

    service = TestBed.inject(BescheidService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('init', () => {
    it('should init state', () => {
      service.setActiveStep(2);

      service.init();

      expect(service.getWizard()).toBeObservable(singleCold(createInitialWizard()));
      expect(service.getActiveStep()).toBeObservable(singleCold(BescheidWizardStep.AntragBescheiden));
      expect(service.getBescheidCreated()).toBeObservable(singleCold(false));
      expect(service.getBescheidDocument()).toBeObservable(singleCold(createEmptyBescheidDocument()));
    });
  });

  describe('exit', () => {
    beforeEach(() => {
      service._clearUploadedFiles = jest.fn();
    });

    it('should reload bescheid list', () => {
      service.exit();

      expect(bescheidListResourceService.refresh).toHaveBeenCalled();
    });

    it('should reload postfach list', () => {
      service.exit();

      expect(postfachService._setPostfachMailOnReload).toHaveBeenCalled();
    });

    it('should clear uploaded files', () => {
      service.exit();

      expect(service._clearUploadedFiles).toHaveBeenCalled();
    });
  });

  describe('skipBescheidCreation', () => {
    beforeEach(() => {
      service.deleteBescheidAndCompleteVorgang = jest.fn().mockReturnValue(of(commandStateResource));
    });

    it('should complete vorgang', () => {
      service.skipBescheidCreation(vorgangWithEingangResource, null);

      expect(vorgangCommandService.abschliessen).toHaveBeenCalledWith(vorgangWithEingangResource);
    });

    it('should NOT complete vorgang', () => {
      service.skipBescheidCreation(vorgangWithEingangResource, bescheidResource);

      expect(vorgangCommandService.abschliessen).not.toHaveBeenCalled();
    });

    it('should delete bescheid and complete vorgang', () => {
      service.skipBescheidCreation(vorgangWithEingangResource, bescheidResource).subscribe();

      expect(service.deleteBescheidAndCompleteVorgang).toHaveBeenCalledWith(vorgangWithEingangResource);
    });

    it('should return command', () => {
      const command$: Observable<StateResource<CommandResource>> = service.skipBescheidCreation(
        vorgangWithEingangResource,
        bescheidResource,
      );

      expect(command$).toBeObservable(singleColdCompleted(commandStateResource));
    });
  });

  describe('deleteBescheidAndCompleteVorgang', () => {
    beforeEach(() => {
      service.deleteBescheid = jest.fn().mockReturnValue(of(createCommandStateResource));
      vorgangCommandService.abschliessen.mockReturnValue(EMPTY);
    });

    it('should complete vorgang', () => {
      service.deleteBescheidAndCompleteVorgang(vorgangWithEingangResource).subscribe();

      expect(vorgangCommandService.abschliessen).toHaveBeenCalledWith(vorgangWithEingangResource);
    });

    it('should delete bescheid', () => {
      vorgangCommandService.abschliessen.mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

      service.deleteBescheidAndCompleteVorgang(vorgangWithEingangResource).subscribe();

      expect(service.deleteBescheid).toHaveBeenCalled();
    });

    it('should NOT delete bescheid on loading', () => {
      vorgangCommandService.abschliessen.mockReturnValue(of(createEmptyStateResource(true)));

      service.deleteBescheidAndCompleteVorgang(vorgangWithEingangResource).subscribe();

      expect(service.deleteBescheid).not.toHaveBeenCalled();
    });

    it('should NOT delete bescheid on error', () => {
      vorgangCommandService.abschliessen.mockReturnValue(of(createErrorStateResource(createProblemDetail())));

      service.deleteBescheidAndCompleteVorgang(vorgangWithEingangResource).subscribe();

      expect(service.deleteBescheid).not.toHaveBeenCalled();
    });

    it('should return vorgang abschliessen command', () => {
      const command: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      vorgangCommandService.abschliessen.mockReturnValue(of(command));

      const command$: Observable<StateResource<CommandResource>> =
        service.deleteBescheidAndCompleteVorgang(vorgangWithEingangResource);

      expect(command$).toBeObservable(singleColdCompleted(command));
    });
  });

  describe('delete bescheid', () => {
    it('should create command', () => {
      service.deleteBescheid();

      expect(bescheidResourceService.delete).toHaveBeenCalled();
    });

    it('should return command', () => {
      const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      bescheidResourceService.delete.mockReturnValue(of(commandStateResource));

      const createdCommand$: Observable<StateResource<CommandResource>> = service.deleteBescheid();

      expect(createdCommand$).toBeObservable(singleColdCompleted(commandStateResource));
    });
  });

  describe('loadFiles', () => {
    beforeEach(() => {
      service.loadBescheidDocument = jest.fn();
      service.loadAttachments = jest.fn();
    });

    it('should bescheid document', () => {
      service.loadFiles(bescheidResource);

      expect(service.loadBescheidDocument).toHaveBeenCalledWith(bescheidResource);
    });

    it('should load attachments', () => {
      service.loadFiles(bescheidResource);

      expect(service.loadAttachments).toHaveBeenCalledWith(bescheidResource);
    });
  });

  describe('create bescheid document', () => {
    beforeEach(() => {
      service.doCreateBescheidDocument = jest.fn().mockReturnValue(EMPTY);
      service.handleCreateBescheidDocumentResponse = jest.fn();
    });

    it('should emit bescheid document', () => {
      service.createBescheidDocument(bescheidResource);

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({ ...createEmptyBescheidDocument(), create: createEmptyStateResource(true) }),
      );
    });

    it('should do create bescheid document', () => {
      service.createBescheidDocument(bescheidResource);

      expect(service.doCreateBescheidDocument).toHaveBeenCalledWith(bescheidResource);
    });

    it('should handle create response on loaded', () => {
      service.doCreateBescheidDocument = jest.fn().mockReturnValue(of(commandStateResource));

      service.createBescheidDocument(bescheidResource);

      expect(service.handleCreateBescheidDocumentResponse).toHaveBeenCalledWith(commandStateResource);
    });

    it('should handle create response on error', () => {
      const commandError: StateResource<CommandResource> = createCommandErrorStateResource();
      service.doCreateBescheidDocument = jest.fn().mockReturnValue(of(commandError));

      service.createBescheidDocument(bescheidResource);

      expect(service.handleCreateBescheidDocumentResponse).toHaveBeenCalledWith(commandError);
    });
  });

  describe('doCreateBescheidDocument', () => {
    it('should create command by props', () => {
      service.doCreateBescheidDocument(bescheidResource);

      expect(commandService.createCommandByProps).toHaveBeenCalledWith({
        resource: bescheidResource,
        linkRel: BescheidLinkRel.CREATE_DOCUMENT,
        command: {
          order: CommandOrder.CREATE_BESCHEID_DOCUMENT,
          body: null,
        },
        snackBarMessage: EMPTY_STRING,
      });
    });
  });

  describe('handleCreateBescheidDocumentResponse', () => {
    const commandStateResource: StateResource<CommandResource> = createStateResource(
      createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
    );

    beforeEach(() => {
      service.loadBescheidDocumentByUri = jest.fn();
      service.emitBescheidDocumentError = jest.fn();
    });

    it('should emit error', () => {
      const commandErrorStateResource: StateResource<CommandResource> = createCommandErrorStateResource();

      service.handleCreateBescheidDocumentResponse(commandErrorStateResource);

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({ ...createEmptyBescheidDocument(), create: commandErrorStateResource } as BescheidDocument),
      );
    });

    it('should emit bescheid document state', () => {
      service.handleCreateBescheidDocumentResponse(commandStateResource);

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({
          ...createEmptyBescheidDocument(),
          documentUri: getEffectedResourceUrl(commandStateResource.resource),
        }),
      );
    });

    it('should load document by uri', () => {
      service.handleCreateBescheidDocumentResponse(commandStateResource);

      expect(service.loadBescheidDocumentByUri).toHaveBeenCalledWith(getEffectedResourceUrl(commandStateResource.resource));
    });
  });

  describe('emitBescheidDocumentError', () => {
    it('should emit', () => {
      const commandError: StateResource<CommandResource> = createCommandErrorStateResource();

      service.emitBescheidDocumentError(commandError);

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({
          ...createEmptyBescheidDocument(),
          upload: { ...createEmptyUploadInProgress(), loading: false, error: commandError.error },
        }),
      );
    });
  });

  describe('loadBescheidDocument', () => {
    beforeEach(() => {
      service.loadBescheidDocumentByUri = jest.fn();
    });

    it('should load by uri', () => {
      const bescheidResource: BescheidResource = createBescheidResource([BescheidLinkRel.BESCHEID_DOCUMENT]);

      service.loadBescheidDocument(bescheidResource);

      expect(service.loadBescheidDocumentByUri).toHaveBeenCalledWith(getUrl(bescheidResource, BescheidLinkRel.BESCHEID_DOCUMENT));
    });

    it('should NOT load by uri', () => {
      service.loadBescheidDocument(bescheidResource);

      expect(service.loadBescheidDocumentByUri).not.toHaveBeenCalled();
    });
  });

  describe('loadBescheidDocumentByUri', () => {
    const resourceUri: ResourceUri = faker.internet.url();

    beforeEach(() => {
      resourceRepository.getResource.mockReturnValue(EMPTY);
      service.loadBescheidDocumentFile = jest.fn();
    });

    it('should get resource', () => {
      service.loadBescheidDocumentByUri(resourceUri);

      expect(resourceRepository.getResource).toHaveBeenCalledWith(resourceUri);
    });

    it('should load bescheid document file', () => {
      const documentResource: DocumentResource = createDocumentResource();
      resourceRepository.getResource.mockReturnValue(of(documentResource));

      service.loadBescheidDocumentByUri(resourceUri);

      expect(service.loadBescheidDocumentFile).toHaveBeenCalledWith(documentResource);
    });
  });

  describe('load bescheid document file', () => {
    const document: DocumentResource = createDocumentResource([DocumentLinkRel.FILE]);
    const binaryFileStateResource: StateResource<BinaryFileResource> = createStateResource(createBinaryFileResource());

    beforeEach(() => {
      binaryFileService.getFile.mockReturnValue(of(binaryFileStateResource));
    });

    it('should call binary file service', () => {
      service.loadBescheidDocumentFile(document);

      expect(binaryFileService.getFile).toHaveBeenCalledWith(getUrl(document, DocumentLinkRel.FILE));
    });

    it('should emit bescheid document state', () => {
      service.loadBescheidDocumentFile(document);

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({
          upload: createEmptyUploadInProgress(),
          create: createEmptyStateResource(),
          resource: binaryFileStateResource.resource,
          documentUri: getUrl(document, LinkRel.Self),
        }),
      );
    });
  });

  describe('loadAttachments', () => {
    const bescheidResourceWithAttachments: BescheidResource = createBescheidResource([BescheidLinkRel.ATTACHMENTS]);

    beforeEach(() => {
      binaryFileService.getFiles.mockReturnValue(EMPTY);
    });

    it('should call binary file service', () => {
      service.loadAttachments(bescheidResourceWithAttachments);

      expect(binaryFileService.getFiles).toHaveBeenCalledWith(bescheidResourceWithAttachments, BescheidLinkRel.ATTACHMENTS);
    });

    it('should NOT call binary file service', () => {
      service.loadAttachments(bescheidResource);

      expect(binaryFileService.getFiles).not.toHaveBeenCalled();
    });

    it('should add files', () => {
      const binaryFileListStateResource: StateResource<BinaryFileListResource> =
        createStateResource(createBinaryFileListResource());
      binaryFileService.getFiles.mockReturnValue(of(binaryFileListStateResource));

      service.loadAttachments(bescheidResourceWithAttachments);

      expect(binaryFileService.addFiles).toHaveBeenCalledWith(
        BESCHEID_UPLOADED_ATTACHMENTS,
        getEmbeddedResources(binaryFileListStateResource, BinaryFileListLinkRel.FILE_LIST),
      );
    });
  });

  describe('upload bescheid document', () => {
    const documentFile: File = createFile();

    beforeEach(() => {
      binaryFileService.uploadFile.mockReturnValue(EMPTY);
      service.handleUploadBescheidDocumentResponse = jest.fn();
    });

    it('should emit bescheid document state', () => {
      service.uploadBescheidDocument(documentFile, bescheidResource);

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({ ...createEmptyBescheidDocument(), upload: { fileName: documentFile.name, loading: true } }),
      );
    });

    it('should call binary file service', () => {
      service.uploadBescheidDocument(documentFile, bescheidResource);

      expect(binaryFileService.uploadFile).toHaveBeenCalledWith(
        bescheidResource,
        BescheidLinkRel.UPLOAD_BESCHEID_FILE,
        documentFile,
        false,
      );
    });

    it('should handle upload response', () => {
      const binaryFileStateResource: StateResource<BinaryFileResource> = createStateResource(createBinaryFileResource());
      binaryFileService.uploadFile.mockReturnValue(of(binaryFileStateResource));

      service.uploadBescheidDocument(documentFile, bescheidResource);

      expect(service.handleUploadBescheidDocumentResponse).toHaveBeenCalledWith(bescheidResource, binaryFileStateResource);
    });
  });

  describe('handleUploadBescheidDocumentResponse', () => {
    beforeEach(() => {
      service.emitBescheidDocumentError = jest.fn();
      service.createBescheidDocumentFromFile = jest.fn();
    });

    it('should emit error', () => {
      const errorStateResource: StateResource<BinaryFileResource> = createErrorStateResource(createProblemDetail());

      service.handleUploadBescheidDocumentResponse(bescheidResource, errorStateResource);

      expect(service.emitBescheidDocumentError).toHaveBeenCalledWith(errorStateResource);
    });

    it('should create bescheid document from file', () => {
      const binaryFileStateResource: StateResource<BinaryFileResource> = createStateResource(createBinaryFileResource());

      service.handleUploadBescheidDocumentResponse(bescheidResource, binaryFileStateResource);

      expect(service.createBescheidDocumentFromFile).toHaveBeenCalledWith(bescheidResource, binaryFileStateResource.resource);
    });
  });

  describe('create bescheid document from file', () => {
    const bescheid: BescheidResource = createBescheidResource();
    const binaryFile: BinaryFileResource = createBinaryFileResource();

    const createCommandProps: CreateCommandProps = createCreateCommandProps();
    let buildCreateBescheidDocumentFromFilePropsMock = buildCreateBescheidDocumentFromFileProps as jest.Mock;

    const commandStateResource: StateResource<CommandResource> = createCommandStateResource();

    beforeEach(() => {
      buildCreateBescheidDocumentFromFilePropsMock.mockReturnValue(createCommandProps);
      commandService.createCommandByProps.mockReturnValue(of(commandStateResource));
    });

    it('should call command sevice', () => {
      service.createBescheidDocumentFromFile(bescheid, binaryFile);

      expect(commandService.createCommandByProps).toHaveBeenCalledWith(createCommandProps);
    });

    it('should build create command document from file props', () => {
      service.createBescheidDocumentFromFile(bescheid, binaryFile);

      expect(buildCreateBescheidDocumentFromFilePropsMock).toHaveBeenCalledWith(bescheid, binaryFile);
    });

    it('should call handle create bescheid document from file response', () => {
      service.handleCreateBescheidDocumentFromFileResponse = jest.fn();

      service.createBescheidDocumentFromFile(bescheid, binaryFile);

      expect(service.handleCreateBescheidDocumentFromFileResponse).toHaveBeenCalledWith(commandStateResource, binaryFile);
    });
  });

  describe('handleCreateBescheidDocumentFromFileResponse', () => {
    const commandStateResource: StateResource<CommandResource> = createStateResource(
      createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
    );

    beforeEach(() => {
      service.emitBescheidDocumentError = jest.fn();
    });

    it('should emit error', () => {
      const errorStateResource: StateResource<CommandResource> = createErrorStateResource(createProblemDetail());

      service.handleCreateBescheidDocumentFromFileResponse(errorStateResource, createBinaryFileResource());

      expect(service.emitBescheidDocumentError).toHaveBeenCalledWith(errorStateResource);
    });

    it('should NOT emit error', () => {
      const binaryFileResource: BinaryFileResource = createBinaryFileResource();

      service.handleCreateBescheidDocumentFromFileResponse(commandStateResource, binaryFileResource);

      expect(service.emitBescheidDocumentError).not.toHaveBeenCalled();
    });

    it('should emit bescheid document state', () => {
      const binaryFileResource: BinaryFileResource = createBinaryFileResource();

      service.handleCreateBescheidDocumentFromFileResponse(commandStateResource, binaryFileResource);

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({
          ...createEmptyBescheidDocument(),
          documentUri: getEffectedResourceUrl(commandStateResource.resource),
          upload: createEmptyUploadInProgress(),
          resource: binaryFileResource,
        }),
      );
    });

    it('should NOT emit bescheid document state', () => {
      const errorStateResource: StateResource<CommandResource> = createErrorStateResource(createProblemDetail());

      service.handleCreateBescheidDocumentFromFileResponse(errorStateResource, createBinaryFileResource());

      expect(service.getBescheidDocument()).toBeObservable(singleCold(createEmptyBescheidDocument()));
    });
  });

  describe('createBescheid', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
    const command: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);
    const commandStateResource: StateResource<CommandResource> = createStateResource(command);

    beforeEach(() => {
      service.updateBescheidDraft = jest.fn();
    });

    beforeEach(() => {
      facade.getBescheidCommand.mockReturnValue(of(commandStateResource));
      (service as any).bescheidResourceService.loadByResourceUri = jest.fn();
    });

    it('should call facade', () => {
      service.createBescheid(vorgangWithEingang).subscribe();

      expect(facade.createBescheidDraft).toHaveBeenCalledWith(vorgangWithEingang, {
        order: CommandOrder.CREATE_BESCHEID,
        body: null,
      });
    });

    it('should update bescheid draft', () => {
      service.createBescheid(vorgangWithEingang).subscribe();

      expect(service.updateBescheidDraft).toHaveBeenCalledWith(commandStateResource.resource);
    });
  });

  describe('updateBescheidDraft', () => {
    const command: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);

    beforeEach(() => {
      (service as any).bescheidResourceService.loadByResourceUri = jest.fn();
    });

    it('should load resource by uri', () => {
      service.updateBescheidDraft(command);

      expect((service as any).bescheidResourceService.loadByResourceUri).toHaveBeenCalledWith(
        getUrl(command, CommandLinkRel.EFFECTED_RESOURCE),
      );
    });

    it('should emit bescheid created', () => {
      service.updateBescheidDraft({ ...command, order: CommandOrder.CREATE_BESCHEID });

      expect(service.getBescheidCreated()).toBeObservable(singleCold(true));
    });
  });

  describe('updateBescheid', () => {
    const bescheid: Bescheid = createBescheid();
    const commandResource: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);
    const commandStateResource: StateResource<CommandResource> = createStateResource(commandResource);

    beforeEach(() => {
      service.doUpdateBescheid = jest.fn().mockReturnValue(EMPTY);
      service.updateBescheidDraft = jest.fn().mockReturnValue(EMPTY);
    });

    it('should do update bescheid', () => {
      service.updateBescheid(bescheidResource, bescheid);

      expect(service.doUpdateBescheid).toHaveBeenCalledWith(bescheidResource, bescheid);
    });

    it('should update bescheid draft', () => {
      service.doUpdateBescheid = jest.fn().mockReturnValue(of(commandStateResource));

      service.updateBescheid(bescheidResource, bescheid).subscribe();

      expect(service.updateBescheidDraft).toHaveBeenCalledWith(commandResource);
    });
  });

  describe('do update bescheid', () => {
    const bescheid: Bescheid = createBescheid();
    const bescheidResource: BescheidResource = createBescheidResource();

    const createCommandProps: CreateCommandProps = createCreateCommandProps();
    let buildUpdateBescheidCommandPropsMock: jest.Mock = buildUpdateBescheidCommandProps as jest.Mock;

    beforeEach(() => {
      buildUpdateBescheidCommandPropsMock.mockReturnValue(createCommandProps);
      commandService.createCommandByProps.mockClear();
      commandService.createCommandByProps.mockReturnValue(of(createCommandStateResource()));
    });

    it('should build update bescheid command props', () => {
      service.doUpdateBescheid(bescheidResource, bescheid);

      expect(buildUpdateBescheidCommandPropsMock).toHaveBeenCalledWith(bescheidResource, bescheid);
    });

    it('should call command service', () => {
      service.doUpdateBescheid(bescheidResource, bescheid).subscribe();

      expect(commandService.createCommandByProps).toHaveBeenCalledWith(createCommandProps);
    });
  });

  describe('sendBescheidManually', () => {
    beforeEach(() => {
      service.sendBescheid = jest.fn().mockReturnValue(EMPTY);
    });

    it('should send bescheid', () => {
      service.sendBescheidManually().subscribe();

      expect(service.sendBescheid).toHaveBeenCalledWith(BescheidLinkRel.BESCHEIDEN);
    });

    it('should return send bescheid command', () => {
      const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      service.sendBescheid = jest.fn().mockReturnValue(of(commandStateResource));

      const sendCommandStateResource$: Observable<StateResource<CommandResource>> = service.sendBescheidManually();

      expect(sendCommandStateResource$).toBeObservable(singleColdCompleted(commandStateResource));
    });
  });

  describe('sendBescheid', () => {
    beforeEach(() => {
      service.getBescheidDraft = jest.fn().mockReturnValue(of(createStateResource(bescheidResource)));
      commandService.createCommandByProps.mockReturnValue(EMPTY);
    });

    it('should get bescheid draft', () => {
      service.sendBescheidManually().subscribe();

      expect(service.getBescheidDraft).toHaveBeenCalled();
    });

    it('should create command', () => {
      service.sendBescheid(BescheidLinkRel.BESCHEIDEN).subscribe();

      expect(commandService.createCommandByProps).toHaveBeenCalledWith({
        resource: bescheidResource,
        linkRel: BescheidLinkRel.BESCHEIDEN,
        command: {
          order: CommandOrder.SEND_BESCHEID,
          body: null,
        },
        snackBarMessage: EMPTY_STRING,
      } as CreateCommandProps);
    });

    it('should return send bescheid command', () => {
      const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      commandService.createCommandByProps = jest.fn().mockReturnValue(of(commandStateResource));

      const sendCommandStateResource$: Observable<StateResource<CommandResource>> = service.sendBescheid(
        BescheidLinkRel.BESCHEIDEN,
      );

      expect(sendCommandStateResource$).toBeObservable(singleColdCompleted(commandStateResource));
    });
  });

  describe('sendBescheidMessage', () => {
    beforeEach(() => {
      service.getBescheidDraft = jest.fn().mockReturnValue(createStateResource(bescheidResource));
      service.sendBescheid = jest.fn().mockReturnValue(EMPTY);
    });

    it('should send bescheid', () => {
      service.sendBescheidMessage().subscribe();

      expect(service.sendBescheid).toHaveBeenCalledWith(BescheidLinkRel.BESCHEIDEN_UND_SENDEN);
    });

    it('should return send bescheid command', () => {
      const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      service.sendBescheid = jest.fn().mockReturnValue(of(commandStateResource));

      const sendCommandStateResource$: Observable<StateResource<CommandResource>> = service.sendBescheidMessage();

      expect(sendCommandStateResource$).toBeObservable(singleColdCompleted(commandStateResource));
    });
  });

  describe('setActiveStep', () => {
    beforeEach(() => {
      service._clearUploadedFiles = jest.fn();
    });

    it('should emit changed active step', () => {
      service.setActiveStep(BescheidWizardStep.DokumenteHochladen);

      expect(service.getActiveStep()).toBeObservable(singleCold(BescheidWizardStep.DokumenteHochladen));
    });

    it('should clear uploaded files', () => {
      service.setActiveStep(BescheidWizardStep.DokumenteHochladen);

      expect(service._clearUploadedFiles).toHaveBeenCalled();
    });
  });

  describe('getBescheidDraft', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();
    const bescheidDraftStateResource: StateResource<BescheidResource> = createStateResource(bescheidDraft);

    it('should call resource service', () => {
      bescheidResourceService.get = jest.fn();

      service.getBescheidDraft();

      expect(bescheidResourceService.get).toHaveBeenCalled();
    });

    it('should return value', () => {
      bescheidResourceService.get = jest.fn().mockReturnValue(singleCold(bescheidDraftStateResource));

      const bescheidStateResource$: Observable<StateResource<BescheidResource>> = service.getBescheidDraft();

      expect(bescheidStateResource$).toBeObservable(singleCold(bescheidDraftStateResource));
    });
  });

  describe('setNachrichtEmpfaenger', () => {
    it('should change wizard state', () => {
      const empfaenger: string = faker.person.fullName();

      service.setNachrichtEmpfaenger(empfaenger);

      expect(service.getWizard()).toBeObservable(singleCold({ ...createInitialWizard(), empfaenger }));
    });
  });

  describe('lockBescheidSending', () => {
    it('should change wizard state', () => {
      service.lockBescheidSending();

      expect(service.getWizard()).toBeObservable(singleCold({ ...createInitialWizard(), canBeSend: false }));
    });
  });

  describe('unlockBescheidSending', () => {
    it('should change wizard state', () => {
      service.unlockBescheidSending();

      expect(service.getWizard()).toBeObservable(singleCold({ ...createInitialWizard(), canBeSend: true }));
    });
  });

  describe('finishAddingBescheidDocument', () => {
    it('should update state', () => {
      const bescheidDocument: BescheidDocument = createBescheidDocument();
      service._bescheidDocument$.next(bescheidDocument);

      service.finishAddingBescheidDocument();

      expect(service.getBescheidDocument()).toBeObservable(
        singleCold({
          ...bescheidDocument,
          create: createEmptyStateResource(),
          upload: createEmptyUploadInProgress(),
        } as BescheidDocument),
      );
    });
  });

  describe('clear uploaded files', () => {
    it('should call binary files service', () => {
      service._clearUploadedFiles();

      expect(binaryFileService.clearUploadedFiles).toHaveBeenCalledWith(BESCHEID_UPLOADED_ATTACHMENTS);
    });
  });

  describe('get bescheid list', () => {
    it('should call bescheid list resource service', () => {
      service.getBescheidList();

      expect(bescheidListResourceService.getList).toHaveBeenCalled();
    });

    it('should return value', () => {
      const bescheidList: BescheidListResource = createBescheidListResource();
      const bescheidListStateResource: StateResource<BescheidListResource> = createStateResource(bescheidList);
      bescheidListResourceService.getList = jest.fn().mockReturnValue(singleCold(bescheidListStateResource));

      const command$: Observable<StateResource<BescheidListResource>> = service.getBescheidList();

      expect(command$).toBeObservable(singleCold(bescheidListStateResource));
    });
  });

  describe('get bescheid document by uri', () => {
    const resourceUri: ResourceUri = faker.internet.url();
    const document: DocumentResource = createDocumentResource();

    beforeEach(() => {
      resourceRepository.getResource.mockReturnValue(of(document));
    });

    it('should call repository', () => {
      service.getBescheidDocumentByUri(resourceUri);

      expect(resourceRepository.getResource).toHaveBeenCalledWith(resourceUri);
    });

    it('should return value', () => {
      const documentStateResource$: Observable<StateResource<DocumentResource>> = service.getBescheidDocumentByUri(resourceUri);

      expect(documentStateResource$).toBeObservable(
        cold('(ab|)', { a: createEmptyStateResource(true), b: createStateResource(document) }),
      );
    });
  });

  describe('get last bescheid', () => {
    const bescheid: BescheidResource = createBescheidResource();
    const bescheide: BescheidResource[] = [bescheid];

    let sortByGermanDateStrSpy: jest.SpyInstance;
    let getItemsSpy: jest.SpyInstance;

    beforeEach(() => {
      service.filterBySentStatus = jest.fn().mockReturnValue(bescheide);
      sortByGermanDateStrSpy = jest.spyOn(DateUtil, 'sortByGermanDateStr').mockReturnValue(bescheide);
      getItemsSpy = bescheidListResourceService.getItems = jest.fn().mockReturnValue(of(bescheide));
    });

    it('should get items', () => {
      service.getLastBescheid().subscribe();

      expect(getItemsSpy).toHaveBeenCalled();
    });

    it('should filter by sent status', () => {
      service.getLastBescheid().subscribe();

      expect(service.filterBySentStatus).toHaveBeenCalledWith(bescheide);
    });

    it('should sort by beschieden am', () => {
      service.getLastBescheid().subscribe();

      expect(sortByGermanDateStrSpy).toHaveBeenCalledWith(bescheide, expect.any(Function));
    });

    it('should return first element', () => {
      const lastBescheid$: Observable<BescheidResource> = service.getLastBescheid();

      expect(lastBescheid$).toBeObservable(singleColdCompleted(bescheid));
    });
  });

  describe('filter by sent status', () => {
    it('should keep entry with sent status', () => {
      const bescheidWithSentStatus: BescheidResource = {
        ...createBescheidResource(),
        status: BescheidStatus.SENT,
      };

      const filteredBescheide: BescheidResource[] = service.filterBySentStatus([bescheidWithSentStatus]);

      expect(filteredBescheide.length).toBe(1);
    });

    it('should filter entry with draft status', () => {
      const bescheidWithDraftStatus: BescheidResource = {
        ...createBescheidResource(),
        status: BescheidStatus.DRAFT,
      };

      const filteredBescheide: BescheidResource[] = service.filterBySentStatus([bescheidWithDraftStatus]);

      expect(filteredBescheide.length).toBe(0);
    });

    it('should return empty array', () => {
      const resultdBescheide: BescheidResource[] = service.filterBySentStatus(null);

      expect(resultdBescheide).toEqual([]);
    });
  });

  describe('exist bescheid', () => {
    const bescheid: BescheidResource = createBescheidResource();
    const bescheide: BescheidResource[] = [bescheid];
    const bescheidListStateResource: StateResource<BescheidListResource> = createStateResource(
      createBescheidListResource(bescheide),
    );
    let getItemsSpy: jest.SpyInstance;

    beforeEach(() => {
      service.getBescheidList = jest.fn().mockReturnValue(of(bescheidListStateResource));
      service.filterBySentStatus = jest.fn().mockReturnValue(bescheide);
      getItemsSpy = bescheidListResourceService.getItems = jest.fn().mockReturnValue(of(bescheide));
    });

    it('should get items', () => {
      service.existBescheid().subscribe();

      expect(getItemsSpy).toHaveBeenCalled();
    });

    it('should filter by sent status', () => {
      service.existBescheid().subscribe();

      expect(service.filterBySentStatus).toHaveBeenCalledWith(bescheide);
    });

    it('should return true if at least one bescheid exists', () => {
      const existBescheid$: Observable<boolean> = service.existBescheid();

      expect(existBescheid$).toBeObservable(singleColdCompleted(true));
    });

    it('should return false if no bescheide exists', () => {
      service.filterBySentStatus = jest.fn().mockReturnValue([]);

      const existBescheid$: Observable<boolean> = service.existBescheid();

      expect(existBescheid$).toBeObservable(singleColdCompleted(false));
    });
  });
});
