/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  BlobWithFileName,
  ContentType,
  GetRequestOptions,
  HttpErrorHandler,
  HttpHeader,
  ListResource,
} from '@alfa-client/tech-shared';
import { mock, mockClass, useFromMock } from '@alfa-client/test-utils';
import { HttpClient, HttpHeaders, HttpResponse } from '@angular/common/http';
import { faker } from '@faker-js/faker';
import { Resource, ResourceFactory, ResourceUri, getUrl } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { InjectorService } from 'libs/tech-shared/src/lib/injector/injector.service';
import { DummyLinkRel } from 'libs/tech-shared/test/dummy';
import { createHttpResponse } from 'libs/tech-shared/test/http';
import { singleCold } from 'libs/tech-shared/test/marbles';
import { createDummyListResource, createDummyResource } from 'libs/tech-shared/test/resource';
import { Observable, of } from 'rxjs';
import { createBinaryFileResource, createBlob, createFile, createGetRequestOptions } from '../../test/binary-file';
import { BinaryFileLinkRel } from './binary-file.linkrel';
import { BinaryFileResource } from './binary-file.model';
import { BinaryFileRepository } from './binary-file.repository';

import * as HttpUtil from 'libs/tech-shared/src/lib/http.util';

describe('BinaryFileRepository', () => {
  let repository: BinaryFileRepository;

  const httpClient = mock(HttpClient);
  const resourceFactory = mock(ResourceFactory);
  const resourceWrapper = { get: jest.fn() };

  beforeEach(() => {
    mockInterceptor();

    repository = new BinaryFileRepository(useFromMock(httpClient), useFromMock(resourceFactory));

    resourceFactory.from.mockReturnValue(resourceWrapper);
    resourceFactory.fromId.mockReturnValue(resourceWrapper);
  });

  function mockInterceptor(): void {
    mockClass(InjectorService).injector = <any>{ get: () => useFromMock(mock(HttpErrorHandler)) };
  }

  it('should be created', () => {
    expect(repository).toBeTruthy();
  });

  describe('upload file new', () => {
    const dummyLinkRel: string = DummyLinkRel.DUMMY;
    const dummyResource: Resource = createDummyResource([dummyLinkRel]);
    const uri: ResourceUri = getUrl(dummyResource, dummyLinkRel);

    const file: File = createFile();
    const binaryFileResource: BinaryFileResource = createBinaryFileResource();
    const httpResponse: HttpResponse<Object> = createHttpResponse();

    const blob: Blob = createBlob();

    beforeEach(() => {
      httpClient.post.mockReturnValue(of(httpResponse));
      repository.getFile = jest.fn().mockReturnValue(of(binaryFileResource));
    });

    it('should call post on http client', () => {
      const formData: FormData = new FormData();
      formData.append('file', blob, file.name);

      repository.uploadFileNew(uri, <File>blob);

      expect(httpClient.post).toHaveBeenCalledWith(getUrl(dummyResource, dummyLinkRel), formData, {
        observe: 'response',
      });
    });

    it('should call get file', () => {
      repository.uploadFileNew(uri, <File>blob).subscribe();

      expect(repository.getFile).toHaveBeenCalledWith(httpResponse.headers.get(HttpHeader.LOCATION));
    });

    it('should return binary file', () => {
      repository.getFile = jest.fn().mockReturnValue(singleCold(binaryFileResource));

      const result: Observable<BinaryFileResource> = repository.uploadFileNew(uri, <File>blob);

      expect(result).toBeObservable(singleCold(binaryFileResource));
    });
  });

  describe('uploadFile', () => {
    const dummyLinkRel: string = DummyLinkRel.DUMMY;
    const dummyResource: Resource = createDummyResource([dummyLinkRel]);
    const blob: Blob = createBlob();

    beforeEach(() => {
      httpClient.post.mockReturnValue(of({}));
    });

    it('should call POST on httpClient with uri and data', () => {
      const formData: FormData = new FormData();
      formData.append('file', blob, 'filename');

      repository.uploadFile(getUrl(dummyResource, dummyLinkRel), <File>blob);

      expect(httpClient.post).toHaveBeenCalledWith(getUrl(dummyResource, dummyLinkRel), formData, {
        observe: 'response',
      });
    });

    it('and return result', () => {
      const result: Observable<HttpResponse<Object>> = repository.uploadFile(getUrl(dummyResource, dummyLinkRel), <File>blob);

      expect(result).not.toBeNull();
    });
  });

  describe('download', () => {
    const blob = {};
    const binaryFileResource: BinaryFileResource = createBinaryFileResource([BinaryFileLinkRel.DOWNLOAD]);

    beforeEach(() => {
      httpClient.get.mockReturnValue(hot('a', { a: blob }));
    });

    it('should call httpClient', () => {
      const requestOptions = {};
      repository.buildRequestOptions = jest.fn();
      (<any>repository.buildRequestOptions).mockReturnValue(requestOptions);

      repository.download(binaryFileResource);

      expect(httpClient.get).toHaveBeenCalledWith(getUrl(binaryFileResource, BinaryFileLinkRel.DOWNLOAD), requestOptions);
    });

    it('should return value', () => {
      const result = repository.download(binaryFileResource);

      expect(result).toBeObservable(cold('b', { b: blob }));
    });

    describe('buildRequestOptions', () => {
      it('should return httpHeaders', () => {
        const result: GetRequestOptions = repository.buildRequestOptions();

        expect(result.responseType).toEqual('blob');
      });

      it('should return responseType', () => {
        const result: GetRequestOptions = repository.buildRequestOptions();

        expect(result.headers).toEqual(getExpectedRequestOptions());
      });
    });

    function getExpectedRequestOptions(): HttpHeaders {
      let headers = new HttpHeaders();
      headers = headers.set('Accept', [ContentType.APPLICATION_ALL, ContentType.IMAGES_ALL]);
      return headers;
    }
  });

  describe('downloadPdf', () => {
    const blob = {};
    const uri: ResourceUri = faker.internet.url();

    beforeEach(() => {
      httpClient.get.mockReturnValue(hot('a', { a: blob }));
    });

    it('should call httpClient', () => {
      const requestOptions = {};
      repository.buildPdfRequestOptions = jest.fn();
      (<any>repository.buildPdfRequestOptions).mockReturnValue(requestOptions);

      repository.downloadPdf(uri);

      expect(httpClient.get).toHaveBeenCalledWith(uri, requestOptions);
    });

    it('should return value', () => {
      const result = repository.downloadPdf(uri);

      expect(result).toBeObservable(cold('b', { b: blob }));
    });

    describe('buildPdfRequestOptions', () => {
      it('should return httpHeaders', () => {
        const result: GetRequestOptions = repository.buildPdfRequestOptions();

        expect(result.responseType).toEqual('blob');
      });

      it('should return responseType', () => {
        const result: GetRequestOptions = repository.buildPdfRequestOptions();

        expect(result.headers).toEqual(getExpectedRequestOptions());
      });
    });

    function getExpectedRequestOptions(): HttpHeaders {
      return new HttpHeaders().set(HttpHeader.ACCEPT, [ContentType.APPLICATION_PDF, ContentType.APPLICATION_JSON]);
    }
  });

  describe('get file', () => {
    const uri: ResourceUri = faker.internet.url();

    it('should call repository factory', () => {
      repository.getFile(uri);

      expect(resourceFactory.fromId).toHaveBeenCalledWith(uri);
    });

    it('should call repository wrapper', () => {
      repository.getFile(uri);

      expect(resourceWrapper.get).toHaveBeenCalled();
    });
  });

  describe('get files', () => {
    const resource: Resource = createDummyResource([DummyLinkRel.DUMMY]);
    const listResource: ListResource = createDummyListResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(cold('a', { a: listResource }));
    });

    it('should call resource factory', () => {
      repository.getFiles(resource, DummyLinkRel.DUMMY);

      expect(resourceFactory.from).toHaveBeenCalledWith(resource);
    });

    it('should call resource wrapper get with linkRel', () => {
      repository.getFiles(resource, DummyLinkRel.DUMMY);

      expect(resourceWrapper.get).toHaveBeenCalledWith(DummyLinkRel.DUMMY);
    });
  });

  describe('downloadArchive', () => {
    const blob: Blob = createBlob();
    const uri: ResourceUri = faker.internet.url();

    const blobWithFileName: BlobWithFileName = { blob, fileName: faker.person.firstName() };

    beforeEach(() => {
      httpClient.get.mockReturnValue(hot('a', { a: blob }));
      jest.spyOn(HttpUtil, 'buildBlobWithFileName').mockReturnValue(blobWithFileName);
    });

    it('should call httpClient', () => {
      const requestOptions: GetRequestOptions = createGetRequestOptions();
      repository.buildRequestOptionsForArchive = jest.fn().mockReturnValue(requestOptions);

      repository.downloadArchive(uri);

      expect(httpClient.get).toHaveBeenCalledWith(uri, requestOptions);
    });

    it('should return value', () => {
      const result: Observable<BlobWithFileName> = repository.downloadArchive(uri);

      expect(result).toBeObservable(cold('b', { b: blobWithFileName }));
    });

    describe('buildRequestOptionsForArchive', () => {
      it('should return httpHeaders', () => {
        const result: GetRequestOptions = repository.buildRequestOptionsForArchive();

        expect(result.responseType).toEqual('blob');
      });

      it('should return responseType', () => {
        const result: GetRequestOptions = repository.buildRequestOptionsForArchive();

        expect(result.headers).toEqual(buildExpectedRequestOptions());
      });

      it('should return observe', () => {
        const result: GetRequestOptions = repository.buildRequestOptionsForArchive();

        expect((<any>result).observe).toEqual('response');
      });
    });

    function buildExpectedRequestOptions(): HttpHeaders {
      return new HttpHeaders().set(HttpHeader.ACCEPT, ContentType.APPLICATION_ZIP);
    }
  });
});
