import { BinaryFileResource, UploadFileByIdentifier } from '@alfa-client/binary-file-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormGroupDirective, UntypedFormArray, UntypedFormBuilder, UntypedFormControl } from '@angular/forms';
import { faker } from '@faker-js/faker/.';
import { getUrl } from '@ngxp/rest';
import { createBinaryFileResource, createUploadFile } from 'libs/binary-file-shared/test/binary-file';
import { FileUploadListComponent } from './file-upload-list.component';

describe('FileUploadListComponent', () => {
  let component: FileUploadListComponent;
  let fixture: ComponentFixture<FileUploadListComponent>;

  const fb: UntypedFormBuilder = new UntypedFormBuilder();
  const formGroupDirective: FormGroupDirective = new FormGroupDirective([], []);
  formGroupDirective.form = fb.group({
    attachments: fb.control(null),
  });

  const fileKey: string = '21';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [FileUploadListComponent],
      providers: [
        {
          provide: FormGroupDirective,
          useValue: formGroupDirective,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(FileUploadListComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('set uploaded files', () => {
      it('should update upload items', () => {
        component._updateUploadItems = jest.fn();
        const uploadedFiles: UploadFileByIdentifier = { [fileKey]: createUploadFile() };

        component.uploadedFiles = uploadedFiles;

        expect(component._updateUploadItems).toHaveBeenCalledWith(uploadedFiles);
      });
    });

    describe('ngOnInit', () => {
      it('should set file link controls', () => {
        component.parentFormArrayName = 'attachments';

        component.ngOnInit();

        expect(component._fileLinkControls).toEqual(formGroupDirective.form.get('attachments'));
      });
    });

    describe('_updateUploadItems', () => {
      beforeEach(() => {
        component._addFileUrl = jest.fn();
        component._updateForm = jest.fn();
      });

      it('should set upload items', () => {
        const uploadedFiles: UploadFileByIdentifier = { [fileKey]: createUploadFile() };

        component._updateUploadItems(uploadedFiles);

        expect(component.uploadItems).toEqual(uploadedFiles);
      });

      it('should set file urls', () => {
        const uploadStateResource: StateResource<BinaryFileResource> = createStateResource(createBinaryFileResource());
        const uploadedFiles: UploadFileByIdentifier = { [fileKey]: { ...createUploadFile(), uploadedFile: uploadStateResource } };

        component._updateUploadItems(uploadedFiles);

        expect(component._fileUrls).toEqual([getUrl(uploadStateResource.resource)]);
      });

      it('should NOT add file url on loading', () => {
        const uploadedFiles: UploadFileByIdentifier = { [fileKey]: { uploadedFile: createEmptyStateResource(true) } };

        component._updateUploadItems(uploadedFiles);

        expect(component._fileUrls).toEqual([]);
      });

      it('should update form', () => {
        const uploadedFiles: UploadFileByIdentifier = { [fileKey]: { uploadedFile: createEmptyStateResource(true) } };

        component._updateUploadItems(uploadedFiles);

        expect(component._updateForm).toHaveBeenCalled();
      });
    });

    describe('_addFileUrl', () => {
      beforeEach(() => {
        component._updateForm = jest.fn();
      });

      it('should add file url', () => {
        const binaryFileResource: BinaryFileResource = createBinaryFileResource();

        component._addFileUrl(binaryFileResource);

        expect(component._fileUrls).toEqual([getUrl(binaryFileResource)]);
      });

      it('should update form', () => {
        const binaryFileResource: BinaryFileResource = createBinaryFileResource();

        component._addFileUrl(binaryFileResource);

        expect(component._updateForm).toHaveBeenCalled();
      });
    });

    describe('_updateForm', () => {
      beforeEach(() => {
        component._fileLinkControls = new UntypedFormArray([new UntypedFormControl(faker.internet.url())]);
      });

      it('should clear file control list', () => {
        component._fileUrls = [faker.internet.url()];

        component._updateForm();

        expect(component._fileLinkControls.length).toEqual(1);
      });
      it('should update file control list', () => {
        const fileUrl: string = faker.internet.url();
        component._fileUrls = [fileUrl];

        component._updateForm();

        expect(component._fileLinkControls.controls[0].value).toEqual(fileUrl);
      });
    });

    describe('onDelete', () => {
      const binaryFileResource: BinaryFileResource = createBinaryFileResource();
      const fileUrl1: string = faker.internet.url();
      const fileUrl2: string = getUrl(binaryFileResource);
      const key: string = faker.word.noun();

      beforeEach(() => {
        component.delete.emit = jest.fn();
        component._fileUrls = [fileUrl1, fileUrl2];
        component._updateForm = jest.fn();
      });

      it('should update file urls', () => {
        component.onDelete({ binaryFileResource, key });

        expect(component._fileUrls).toEqual([fileUrl1]);
      });

      it('should emit', () => {
        component.onDelete({ binaryFileResource, key });

        expect(component.delete.emit).toHaveBeenCalledWith(key);
      });

      it('should update form', () => {
        component.onDelete({ binaryFileResource, key });

        expect(component._updateForm).toHaveBeenCalled();
      });
    });
  });
});
