/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { ApiError } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action } from '@ngrx/store';
import { provideMockStore } from '@ngrx/store/testing';
import { cold, hot } from 'jest-marbles';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createHintListResource } from 'libs/hint-shared/test/hint';
import { createApiError } from 'libs/tech-shared/test/error';
import { Observable, of } from 'rxjs';
import { HintListResource } from '../hint.model';
import { HintEffects } from './hint.effects';
import { HintRepository } from './hint.repository';

import * as ApiRootActions from '../../../../api-root-shared/src/lib/+state/api-root.actions';
import * as HintActions from './hint.actions';

describe('HintEffects', () => {
  let actions: Observable<Action>;
  let effects: HintEffects;

  const repository = mock(HintRepository);

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        HintEffects,
        provideMockActions(() => actions),
        provideMockStore(),
        {
          provide: HintRepository,
          useValue: repository,
        },
      ],
    });

    effects = TestBed.inject(HintEffects);
  });

  describe('loadAll', () => {
    const apiRoot: ApiRootResource = createApiRootResource([ApiRootLinkRel.HINTS]);
    const action = ApiRootActions.loadApiRootSuccess({ apiRoot });

    it('should call repository', () => {
      actions = of(action);

      effects.loadAll$.subscribe();

      expect(repository.loadAll).toHaveBeenCalledWith(apiRoot);
    });

    it('should dispatch loadHintListSuccess action', () => {
      const hintList: HintListResource = createHintListResource();
      repository.loadAll.mockReturnValue(of(hintList));

      actions = hot('-a-|', { a: action });

      const expected = hot('-a-|', { a: HintActions.loadHintListSuccess({ hintList }) });
      expect(effects.loadAll$).toBeObservable(expected);
    });

    it('should dispatch loadHintListFailed action', () => {
      const apiError: ApiError = createApiError();
      const error = { error: { error: apiError } };
      const errorResponse = cold('-#', {}, error);
      repository.loadAll = jest.fn(() => errorResponse);

      const expected = cold('--b', { b: HintActions.loadHintListFailed({ apiError }) });
      actions = hot('-a', { a: action });

      expect(effects.loadAll$).toBeObservable(expected);
    });
  });

  describe('loadAll without link', () => {
    const apiRoot: ApiRootResource = createApiRootResource();
    const action = ApiRootActions.loadApiRootSuccess({ apiRoot });

    it('should not call repository', () => {
      actions = of(action);

      effects.loadAll$.subscribe();

      expect(repository.loadAll).not.toHaveBeenCalledWith(apiRoot);
    });
  });
});
