/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootLinkRel } from '@alfa-client/api-root-shared';
import { RouteData } from '@alfa-client/navigation-shared';
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { UrlSegment } from '@angular/router';
import { faker } from '@faker-js/faker';
import { createRouteData } from 'libs/navigation-shared/test/navigation-test-factory';
import { initialState, VorgangState } from './+state/vorgang.reducer';
import {
  ALLE_ROUTE_PARAM,
  buildLinkRel,
  buildLinkRelFromPathSegments,
  buildPathSegmentsFromLocalStorage,
  buildVorgangFilterViewRoutePath,
  buildVorgangListRouteWithVorgangFilter,
  getSearchLinkRel,
  getSearchString,
  getVorgangFilter,
  isVorgangPage,
  ROUTE_PARAM_BY_VORGANG_FILTER,
  ROUTE_PARAM_BY_VORGANG_VIEW,
  SEARCH_ROUTE_PARAM,
} from './vorgang-navigation.util';
import { VorgangFilter, VorgangView } from './vorgang.model';

import * as Storage from '@alfa-client/app-shared';
import * as VorgangNavigationUtil from './vorgang-navigation.util';

jest.mock('@alfa-client/app-shared', () => mockAsEsModule('@alfa-client/app-shared'));

function mockAsEsModule(module: string) {
  return {
    __esModule: true,
    ...jest.requireActual(module),
  };
}

describe('Vorgang Navigation Util', () => {
  afterEach(() => {
    jest.restoreAllMocks();
  });

  describe('getSearchString', () => {
    it('should return searchString', () => {
      const searchString: string = 'searchThisForMe';

      const result = getSearchString(buildCurrentRouteData(searchString));

      expect(result).toBe(searchString);
    });

    it('should return decoded searchString', () => {
      const searchString: string = 'search This For Me';

      const result = getSearchString(buildCurrentRouteData(encodeURIComponent(searchString)));

      expect(result).toBe(searchString);
    });

    it('should return empty string', () => {
      const searchString: string = EMPTY_STRING;

      const result = getSearchString(buildCurrentRouteData(searchString));

      expect(result).toBe(EMPTY_STRING);
    });

    function buildCurrentRouteData(searchString: string): RouteData {
      const urlSegments: UrlSegment[] = [
        <any>{ path: ALLE_ROUTE_PARAM },
        <any>{ path: SEARCH_ROUTE_PARAM },
        <any>{ path: searchString },
      ];
      const queryParameter = { [SEARCH_ROUTE_PARAM]: searchString };
      return { ...createRouteData(), urlSegments, queryParameter };
    }
  });

  describe('getSearchLinkRel', () => {
    it('should return linkrel for my', () => {
      const linkRel: string = getSearchLinkRel(VorgangFilter.MEINE_VORGAENGE);

      expect(linkRel).toBe(ApiRootLinkRel.SEARCH_MEINE);
    });

    it('should return linkrel for all', () => {
      const linkRel: string = getSearchLinkRel(VorgangFilter.ALLE);

      expect(linkRel).toBe(ApiRootLinkRel.SEARCH_ALLE);
    });

    it('should return linkrel for unassigned', () => {
      const linkRel: string = getSearchLinkRel(VorgangFilter.NICHT_ZUGEWIESEN);

      expect(linkRel).toBe(ApiRootLinkRel.SEARCH_UNASSIGNED);
    });
  });

  describe('getVorgangFilter', () => {
    it('should return undefined if not uebersichtsSeite', () => {
      jest.spyOn(VorgangNavigationUtil, 'isUebersichtsSeite').mockReturnValue(false);

      const vorgangFilter: VorgangFilter = getVorgangFilter(createRouteData());

      expect(vorgangFilter).toBeUndefined();
    });

    it('should return value on uebersichtsSeite', () => {
      jest.spyOn(VorgangNavigationUtil, 'isUebersichtsSeite').mockReturnValue(true);

      const vorgangFilter: VorgangFilter = getVorgangFilter(buildWithRouteParam(VorgangFilter.ALLE));

      expect(vorgangFilter).toBe(VorgangFilter.ALLE);
    });
  });

  describe('isViewSelected', () => {
    it('should call getVorgangView', () => {
      const utilSpy = jest.spyOn(VorgangNavigationUtil, 'getVorgangView');
      const routeData: RouteData = createRouteData();

      VorgangNavigationUtil.isViewSelected(routeData, VorgangView.NEU);

      expect(utilSpy).toHaveBeenCalledWith(routeData);
    });

    it('should return true on matching vorgangView', () => {
      const vorgangView: VorgangView = VorgangView.ANGENOMMEN;
      jest.spyOn(VorgangNavigationUtil, 'getVorgangView').mockReturnValue(vorgangView);

      const isSelected: boolean = VorgangNavigationUtil.isViewSelected(createRouteData(), vorgangView);

      expect(isSelected).toBeTruthy();
    });

    it('should return false on NON matching VorgangView', () => {
      const vorgangView: VorgangView = VorgangView.ANGENOMMEN;
      jest.spyOn(VorgangNavigationUtil, 'getVorgangView').mockReturnValue(VorgangView.NEU);

      const isSelected: boolean = VorgangNavigationUtil.isViewSelected(createRouteData(), vorgangView);

      expect(isSelected).toBeFalsy();
    });
  });

  describe('getVorgangView', () => {
    it('should call getRouteUrlSegment', () => {
      const utilSpy = jest.spyOn(VorgangNavigationUtil, 'getRouteUrlSegment');
      const routeData: RouteData = createRouteData();

      VorgangNavigationUtil.getVorgangView(routeData);

      expect(utilSpy).toHaveBeenCalledWith(routeData, 1);
    });

    it('should call isUebersichtsSeite', () => {
      const utilSpy = jest.spyOn(VorgangNavigationUtil, 'isUebersichtsSeite');
      const routeData: RouteData = createRouteData();

      VorgangNavigationUtil.getVorgangView(routeData);

      expect(utilSpy).toHaveBeenCalledWith(routeData);
    });

    it('should return VorgangView.VORGANG_LIST if routeData has no view', () => {
      const urlSegments = [<any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.ALLE] }];
      const routeData: RouteData = { ...createRouteData(), urlSegments };

      const vorgangView: VorgangView = VorgangNavigationUtil.getVorgangView(routeData);

      expect(vorgangView).toBe(VorgangView.VORGANG_LIST);
    });

    it('should return VorgangView.NEU if routeData contains this view', () => {
      const urlSegments = [
        <any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.ALLE] },
        <any>{ path: VorgangNavigationUtil.NEU_ROUTE_PARAM },
      ];
      const routeData: RouteData = { ...createRouteData(), urlSegments };

      const vorgangView: VorgangView = VorgangNavigationUtil.getVorgangView(routeData);

      expect(vorgangView).toBe(0);
    });
  });

  describe('isUebersichtsSeite', () => {
    it('should return true on Alle VorgangFilter', () => {
      const routeData: RouteData = buildWithRouteParam(VorgangFilter.ALLE);

      const isUebersichtsSeite: boolean = VorgangNavigationUtil.isUebersichtsSeite(routeData);

      expect(isUebersichtsSeite).toBeTruthy();
    });

    it('should return true on Meine Vorgänge VorgangFilter', () => {
      const routeData: RouteData = buildWithRouteParam(VorgangFilter.MEINE_VORGAENGE);

      const isUebersichtsSeite: boolean = VorgangNavigationUtil.isUebersichtsSeite(routeData);

      expect(isUebersichtsSeite).toBeTruthy();
    });

    it('should return true on Nicht zugewiesen VorgangFilter', () => {
      const routeData: RouteData = buildWithRouteParam(VorgangFilter.NICHT_ZUGEWIESEN);

      const isUebersichtsSeite: boolean = VorgangNavigationUtil.isUebersichtsSeite(routeData);

      expect(isUebersichtsSeite).toBeTruthy();
    });

    it('should return false on other "vorgang" routeParam', () => {
      const urlSegments: UrlSegment[] = [<any>{ path: 'vorgang' }];
      const routeData: RouteData = { ...createRouteData(), urlSegments };

      const isUebersichtsSeite: boolean = VorgangNavigationUtil.isUebersichtsSeite(routeData);

      expect(isUebersichtsSeite).toBeFalsy();
    });
  });

  describe('buildBackButtonUrl', () => {
    describe('by state if differs from localStorage', () => {
      it('should call buildPathSegmentsFromLocalStorage', () => {
        jest.spyOn(VorgangNavigationUtil, 'isStateEqualLocalStorage').mockReturnValue(false);
        const utilSpy = jest.spyOn(VorgangNavigationUtil, 'buildPathSegmentsFromLocalStorage');

        VorgangNavigationUtil.buildBackButtonUrl(initialState);

        expect(utilSpy).toHaveBeenCalled();
      });

      it('should build by given filter and view', () => {
        jest.spyOn(Storage, 'getViewFromLocalStorage').mockReturnValue(ROUTE_PARAM_BY_VORGANG_VIEW[VorgangView.ANGENOMMEN]);
        jest
          .spyOn(Storage, 'getFilterFromLocalStorage')
          .mockReturnValue(ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.MEINE_VORGAENGE]);

        const url: string = VorgangNavigationUtil.buildBackButtonUrl(initialState);

        expect(url).toBe(`/meine/angenommen`);
      });

      it('should build by given filter only', () => {
        jest.spyOn(Storage, 'getViewFromLocalStorage').mockReturnValue(EMPTY_STRING);
        jest
          .spyOn(Storage, 'getFilterFromLocalStorage')
          .mockReturnValue(ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.MEINE_VORGAENGE]);

        const url: string = VorgangNavigationUtil.buildBackButtonUrl(initialState);

        expect(url).toBe(`/meine`);
      });
    });

    describe('by state if equals localStorage', () => {
      it('should call buildVorgangListRouteWithVorgangFilter', () => {
        jest.spyOn(VorgangNavigationUtil, 'isStateEqualLocalStorage').mockReturnValue(true);
        const utilSpy = jest.spyOn(VorgangNavigationUtil, 'buildVorgangListRouteWithVorgangFilter');

        VorgangNavigationUtil.buildBackButtonUrl(initialState);

        expect(utilSpy).toHaveBeenCalledWith(initialState, initialState.vorgangFilter);
      });

      it('should build by given filter', () => {
        jest.spyOn(Storage, 'getViewFromLocalStorage').mockReturnValue(EMPTY_STRING);
        jest.spyOn(Storage, 'getFilterFromLocalStorage').mockReturnValue('alle');

        const url: string = VorgangNavigationUtil.buildBackButtonUrl(initialState);

        expect(url).toBe('/alle');
      });

      it('should build by given filter and view', () => {
        jest.spyOn(Storage, 'getViewFromLocalStorage').mockReturnValue('angenommen');
        jest.spyOn(Storage, 'getFilterFromLocalStorage').mockReturnValue('meine');

        const url: string = VorgangNavigationUtil.buildBackButtonUrl({
          ...initialState,
          vorgangFilter: VorgangFilter.MEINE_VORGAENGE,
          vorgangView: VorgangView.ANGENOMMEN,
        });

        expect(url).toBe('/meine/angenommen');
      });
    });
  });

  describe('buildVorgangListRoutePathWithFilter', () => {
    it('should call "buildVorgangListRouteWithVorgangFilter"', () => {
      const utilSpy = jest.spyOn(VorgangNavigationUtil, 'buildVorgangListRouteWithVorgangFilter');
      const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;

      VorgangNavigationUtil.buildVorgangListRoutePathWithFilter(initialState, vorgangFilter);

      expect(utilSpy).toHaveBeenCalledWith(initialState, vorgangFilter);
    });
  });

  describe('buildVorgangListRouteWithVorgangFilter', () => {
    it('should build by alle filter and vorgangList view', () => {
      const state: VorgangState = { ...initialState, vorgangView: VorgangView.VORGANG_LIST };
      const routePath: string = buildVorgangListRouteWithVorgangFilter(state, VorgangFilter.ALLE);

      expect(routePath).toEqual('/alle');
    });

    it('should build by meine vorgänge filter and view', () => {
      const state: VorgangState = { ...initialState, vorgangView: VorgangView.NEU };
      const routePath: string = buildVorgangListRouteWithVorgangFilter(state, VorgangFilter.MEINE_VORGAENGE);

      expect(routePath).toEqual('/meine/neu');
    });

    it('should build by filter search view and existing searchString', () => {
      const state: VorgangState = {
        ...initialState,
        vorgangView: VorgangView.SEARCH,
        searchString: 'testSearchString',
      };
      const routePath: string = buildVorgangListRouteWithVorgangFilter(state, VorgangFilter.MEINE_VORGAENGE);

      expect(routePath).toEqual('/meine/search/testSearchString');
    });

    it('should build by filter other view than search and existing searchString', () => {
      const state: VorgangState = {
        ...initialState,
        vorgangView: VorgangView.ANGENOMMEN,
        searchString: 'testSearchString',
      };
      const routePath: string = buildVorgangListRouteWithVorgangFilter(state, VorgangFilter.MEINE_VORGAENGE);

      expect(routePath).toEqual('/meine/angenommen');
    });
  });

  describe('buildLinkRel', () => {
    describe('on non vorgangList View', () => {
      it('return linkRel for Alle Filter', () => {
        const linkRel: string = buildLinkRel(VorgangFilter.ALLE, VorgangView.NEU);

        expect(linkRel).toBe('vorgaenge_neu_all');
      });

      it('return linkRel for Meine Vorgänge Filter', () => {
        const linkRel: string = buildLinkRel(VorgangFilter.MEINE_VORGAENGE, VorgangView.NEU);

        expect(linkRel).toBe('vorgaenge_neu_my');
      });

      it('return linkRel for Nicht zugewiesen Filter', () => {
        const linkRel: string = buildLinkRel(VorgangFilter.NICHT_ZUGEWIESEN, VorgangView.NEU);

        expect(linkRel).toBe('vorgaenge_neu_unassigned');
      });
    });

    describe('on vorgangList View', () => {
      it('return linkRel for Alle Filter', () => {
        const linkRel: string = buildLinkRel(VorgangFilter.ALLE, VorgangView.VORGANG_LIST);

        expect(linkRel).toBe('vorgaenge_all');
      });

      it('return linkRel for Meine Vorgänge Filter', () => {
        const linkRel: string = buildLinkRel(VorgangFilter.MEINE_VORGAENGE, VorgangView.VORGANG_LIST);

        expect(linkRel).toBe('vorgaenge_my');
      });

      it('return linkRel for Nicht Zugewiesen Filter', () => {
        const linkRel: string = buildLinkRel(VorgangFilter.NICHT_ZUGEWIESEN, VorgangView.VORGANG_LIST);

        expect(linkRel).toBe('vorgaenge_unassigned');
      });
    });
  });

  describe('buildLinkRelFromPathSegments', () => {
    it('should return linkRel with view', () => {
      const pathSegments: string[] = ['meine', 'neu'];

      const linkRel: string = buildLinkRelFromPathSegments(pathSegments);

      expect(linkRel).toBe('vorgaenge_neu_my');
    });

    it('should return linkRel without view', () => {
      const pathSegments: string[] = ['meine'];

      const linkRel: string = buildLinkRelFromPathSegments(pathSegments);

      expect(linkRel).toBe('vorgaenge_my');
    });

    it('should return search linkRel', () => {
      const pathSegments: string[] = ['meine', 'search'];

      const linkRel: string = buildLinkRelFromPathSegments(pathSegments);

      expect(linkRel).toBe('search_my');
    });
  });

  describe('buildVorgangFilterViewRoutePath', () => {
    const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;
    const vorgangView: VorgangView = VorgangView.NEU;

    it('should return routePath of view and filter', () => {
      const routePath: string = buildVorgangFilterViewRoutePath(vorgangFilter, vorgangView, EMPTY_STRING);

      expect(routePath).toBe('/alle/neu');
    });

    it('should return routePath of view and filter and searchString', () => {
      const routePath: string = buildVorgangFilterViewRoutePath(vorgangFilter, vorgangView, 'searchString');

      expect(routePath).toBe('/alle/neu/searchString');
    });
  });

  describe('buildPathSegmentsFromLocalStorage', () => {
    it('should build on existing filter only', () => {
      const storageSpy = jest.spyOn(Storage, 'getFilterFromLocalStorage');
      storageSpy.mockReturnValue('meine');

      jest.spyOn(Storage, 'getViewFromLocalStorage').mockReturnValue(EMPTY_STRING);

      const pathSegments: string[] = buildPathSegmentsFromLocalStorage();

      expect(pathSegments[0]).toBe('meine');
    });

    it('should build on existing filter and view', () => {
      const filterSpy = jest.spyOn(Storage, 'getFilterFromLocalStorage');
      filterSpy.mockReturnValue('meine');

      const viewSpy = jest.spyOn(Storage, 'getViewFromLocalStorage');
      viewSpy.mockReturnValue('neu');

      const pathSegments: string[] = buildPathSegmentsFromLocalStorage();

      expect(pathSegments[0]).toBe('meine');
      expect(pathSegments[1]).toBe('neu');
    });

    it('should build on missing filter and view', () => {
      const filterSpy = jest.spyOn(Storage, 'getFilterFromLocalStorage');
      filterSpy.mockReturnValue(EMPTY_STRING);

      const viewSpy = jest.spyOn(Storage, 'getViewFromLocalStorage');
      viewSpy.mockReturnValue(EMPTY_STRING);

      const pathSegments: string[] = buildPathSegmentsFromLocalStorage();

      expect(pathSegments[0]).toBe('alle');
    });
  });

  function buildWithRouteParam(vorgangFilter: VorgangFilter): RouteData {
    const urlSegments: UrlSegment[] = [<any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[vorgangFilter] }];
    return { ...createRouteData(), urlSegments };
  }

  describe('isVorgangPage', () => {
    it('should return true on matching param and segment length', () => {
      const is: boolean = isVorgangPage(buildVorgangPageRouteData());

      expect(is).toBeTruthy();
    });

    function buildVorgangPageRouteData(): RouteData {
      return {
        ...createRouteData(),
        urlSegments: [],
        queryParameter: {
          [VorgangNavigationUtil.VORGANG_WITH_EINGANG_ROUTE_PARAM]: faker.internet.url(),
        },
      };
    }
  });
});
